from __future__ import print_function
import gc
import sys
import time
import numpy as np

try:
    # Matplotlib is not a dependency
    import matplotlib as mpl
    mpl.use('Agg')  # force the antigrain backend
except (ImportError, RuntimeError):
    mpl = None

from ase.units import Bohr
from ase.utils import devnull

from gpaw.mpi import world, distribute_cpus
from gpaw.utilities.tools import tri2full, md5_array, gram_schmidt
from gpaw.band_descriptor import BandDescriptor
from gpaw.grid_descriptor import GridDescriptor
from gpaw.kohnsham_layouts import BandLayouts
from gpaw.hs_operators import MatrixOperator
from gpaw.parameters import InputParameters
from gpaw.xc import XC
from gpaw.setup import SetupData, Setups
from gpaw.lfc import LFC

memstats = False

from gpaw.test.ut_common import shapeopt, TestCase, \
    TextTestRunner, CustomTextTestRunner, defaultTestLoader, \
    initialTestLoader, create_random_atoms, create_parsize_maxbands


p = InputParameters(spinpol=False)
xc = XC(p.xc)
p.setups = dict([(symbol, SetupData(symbol, xc.name)) for symbol in 'HN'])

class UTBandParallelSetup(TestCase):
    """
    Setup a simple band parallel calculation."""

    # Number of bands
    nbands = 36

    # Spin-paired, single kpoint
    nspins = 1
    nibzkpts = 1

    # Strided or blocked groups
    parstride_bands = None

    # Mean spacing and number of grid points per axis (G x G x G)
    h = 1.0 / Bohr
    G = 20

    # Wavefunction data type
    dtype = None

    def setUp(self):
        for virtvar in ['dtype','parstride_bands']:
            assert getattr(self,virtvar) is not None, 'Virtual "%s"!' % virtvar

        parsize_domain, parsize_bands = create_parsize_maxbands(self.nbands, world.size)
        assert self.nbands % parsize_bands == 0
        comms = distribute_cpus(parsize_domain,
                                parsize_bands, self.nspins, self.nibzkpts)
        domain_comm, kpt_comm, band_comm, block_comm = \
            [comms[name] for name in 'dkbK']
        self.block_comm = block_comm

        # Set up band descriptor:
        self.bd = BandDescriptor(self.nbands, band_comm, self.parstride_bands)

        # Set up grid descriptor:
        res, ngpts = shapeopt(100, self.G**3, 3, 0.2)
        cell_c = self.h * np.array(ngpts)
        pbc_c = (True, False, True)
        self.gd = GridDescriptor(ngpts, cell_c, pbc_c, domain_comm, parsize_domain)

        # Create Kohn-Sham layouts for these band and grid descriptors:
        self.ksl = self.create_kohn_sham_layouts()

        # What to do about kpoints?
        self.kpt_comm = kpt_comm

    def tearDown(self):
        del self.bd, self.gd, self.ksl, self.kpt_comm, self.block_comm

    def create_kohn_sham_layouts(self):
        return BandLayouts(self.gd, self.bd, self.block_comm, self.dtype)

    # =================================

    def verify_comm_sizes(self):
        if world.size == 1:
            return
        comm_sizes = tuple([comm.size for comm in [world, self.bd.comm, \
                                                   self.gd.comm, self.kpt_comm]])
        self._parinfo =  '%d world, %d band, %d domain, %d kpt' % comm_sizes
        self.assertEqual((self.nspins*self.nibzkpts) % self.kpt_comm.size, 0)

    def verify_kohn_sham_layouts(self):
        # TODO do more here :)
        self.assertFalse(self.ksl.using_blacs)
        self.assertTrue(self.ksl.bd is self.bd)
        self.assertTrue(self.ksl.gd is self.gd)

    def verify_band_stride_related(self):
        # Verify that (q1+q2)%B-q1 falls in ]-B;Q[ where Q=B//2+1
        for B in range(1,256):
            Q = B//2+1
            #dqs = []
            #for q1 in range(B):
            #    for q2 in range(Q):
            #        dq = (q1+q2)%B-q1
            #        dqs.append(dq)
            #dqs = np.array(dqs)
            q1s = np.arange(B)[:,np.newaxis]
            q2s = np.arange(Q)[np.newaxis,:]
            dqs = (q1s+q2s)%B-q1s
            self.assertEqual(dqs.min(), -B+1)
            self.assertEqual(dqs.max(), Q-1)

    def verify_band_indexing_consistency(self):
        for n in range(self.bd.nbands):
            band_rank, myn = self.bd.who_has(n)
            self.assertEqual(self.bd.global_index(myn, band_rank), n)

        for band_rank in range(self.bd.comm.size):
            for myn in range(self.bd.mynbands):
                n = self.bd.global_index(myn, band_rank)
                self.assertTrue(self.bd.who_has(n) == (band_rank, myn))

    def verify_band_ranking_consistency(self):
        rank_n = self.bd.get_band_ranks()

        for band_rank in range(self.bd.comm.size):
            my_band_indices = self.bd.get_band_indices(band_rank)
            matches = np.argwhere(rank_n == band_rank).ravel()
            self.assertTrue((matches == my_band_indices).all())

            for myn in range(self.bd.mynbands):
                n = self.bd.global_index(myn, band_rank)
                self.assertEqual(my_band_indices[myn], n)


class UTBandParallelSetup_Blocked(UTBandParallelSetup):
    __doc__ = UTBandParallelSetup.__doc__
    parstride_bands = False
    dtype = float

class UTBandParallelSetup_Strided(UTBandParallelSetup):
    __doc__ = UTBandParallelSetup.__doc__
    parstride_bands = True
    dtype = float
# -------------------------------------------------------------------

def record_memory(wait=0.1):
    assert gc.collect() == 0, 'Uncollected garbage!'
    world.barrier()
    time.sleep(wait)
    mem = MemoryStatistics()
    time.sleep(wait)
    world.barrier()
    return mem

def create_memory_info(mem1, mem2, vmkey='VmData'):
    dm = np.array([(mem2-mem1)[vmkey]], dtype=float)
    dm_r = np.empty(world.size, dtype=float)
    world.all_gather(dm, dm_r)
    return dm_r, ','.join(['%8.4f MB' % v for v in dm_r/1024**2.])

# -------------------------------------------------------------------

class UTConstantWavefunctionSetup(UTBandParallelSetup):
    __doc__ = UTBandParallelSetup.__doc__ + """
    The pseudo wavefunctions are constants normalized to their band index."""

    allocated = False
    blocking = None
    async = None
    
    def setUp(self):
        UTBandParallelSetup.setUp(self)
        for virtvar in ['dtype','blocking','async']:
            assert getattr(self,virtvar) is not None, 'Virtual "%s"!' % virtvar

        # Create randomized atoms
        self.atoms = create_random_atoms(self.gd)

        # Do we agree on the atomic positions?
        pos_ac = self.atoms.get_positions()
        pos_rac = np.empty((world.size,)+pos_ac.shape, pos_ac.dtype)
        world.all_gather(pos_ac, pos_rac)
        if (pos_rac-pos_rac[world.rank,...][np.newaxis,...]).any():
            raise RuntimeError('Discrepancy in atomic positions detected.')

        # Create setups for atoms
        self.Z_a = self.atoms.get_atomic_numbers()
        self.setups = Setups(self.Z_a, p.setups, p.basis,
                             p.lmax, xc)

        # Create atomic projector overlaps
        spos_ac = self.atoms.get_scaled_positions() % 1.0
        self.rank_a = self.gd.get_ranks_from_positions(spos_ac)
        self.pt = LFC(self.gd, [setup.pt_j for setup in self.setups],
                      dtype=self.dtype)
        self.pt.set_positions(spos_ac)

        if memstats:
            # Hack to scramble heap usage into steady-state level
            HEAPSIZE = 25 * 1024**2
            for i in range(100):
                data = np.empty(np.random.uniform(0, HEAPSIZE // 8), float)
                del data
            self.mem_pre = record_memory()
            self.mem_alloc = None
            self.mem_test = None

        # Stuff for pseudo wave functions and projections
        if self.dtype == complex:
            self.gamma = 1j**(1.0/self.nbands)
        else:
            self.gamma = 1.0

        self.psit_nG = None
        self.P_ani = None
        self.Qeff_a = None
        self.Qtotal = None

        self.allocate()

    def tearDown(self):
        UTBandParallelSetup.tearDown(self)
        del self.P_ani, self.psit_nG
        del self.pt, self.setups, self.atoms
        if memstats:
            self.print_memory_summary()
            del self.mem_pre, self.mem_alloc, self.mem_test
        self.allocated = False

    def print_memory_summary(self):
        if not memstats:
            raise RuntimeError('No memory statistics were recorded!')

        if world.rank == 0:
            sys.stdout.write('\n')
            sys.stdout.flush()

        dm_r, dminfo = create_memory_info(MemorySingleton(), self.mem_pre)
        if world.rank == 0:
            print('overhead: %s -> %8.4f MB' % (dminfo, dm_r.sum()/1024**2.))

        dm_r, dminfo = create_memory_info(self.mem_pre, self.mem_alloc)
        if world.rank == 0:
            print('allocate: %s -> %8.4f MB' % (dminfo, dm_r.sum()/1024**2.))

        dm_r, dminfo = create_memory_info(self.mem_alloc, self.mem_test)
        if world.rank == 0:
            print('test-use: %s -> %8.4f MB' % (dminfo, dm_r.sum()/1024**2.))

    def allocate(self):
        """
        Allocate constant wavefunctions and their projections according to::

                          /          \     _____    i*phase*(n-m)
           <psi |psi > = ( 1 + Q      ) * V m*n  * e
               n    n'    \     total/

        """
        if self.allocated:
            raise RuntimeError('Already allocated!')

        self.allocate_wavefunctions()
        self.allocate_projections()

        # XXX DEBUG disables projection contributions
        if False:
            for a,P_ni in self.P_ani.items():
                P_ni[:] = 0.
                self.Qeff_a[a] = 0.
            self.Qtotal = 0.
            self.Z_a[:] = 2**63-1
        # XXX DEBUG

        self.Qtotal = np.empty(1, dtype=float)
        self.Qtotal[:] = np.sum([Qeff for Qeff in self.Qeff_a.values()])
        self.gd.comm.sum(self.Qtotal)

        band_indices = np.arange(self.nbands).astype(self.dtype)
        z = self.gamma**band_indices * band_indices**0.5
        self.S0_nn = (1. + self.Qtotal) * np.outer(z.conj(), z)

        self.allocated = True

        if memstats:
            self.mem_alloc = record_memory()

    def allocate_wavefunctions(self):
        """
        Allocate constant pseudo wavefunctions according to::

             ~    ~        _____    i*phase*(n-m)
           <psi |psi > =  V m*n  * e
               n    n'

        """
        if self.allocated:
            raise RuntimeError('Already allocated!')

        # Fill in wave functions
        gpts_c = self.gd.get_size_of_global_array()
        self.psit_nG = self.gd.empty(self.bd.mynbands, self.dtype)
        for myn, psit_G in enumerate(self.psit_nG):
            n = self.bd.global_index(myn)
            # Fill psit_nG: | psit_n > = exp(i*phase*n) * sqrt(n) / sqrt(V)
            psit_G[:] = self.gamma**n * n**0.5 / (self.gd.dv * gpts_c.prod())**0.5

    def allocate_projections(self):
        """
        Construct dummy projection of pseudo wavefunction according to::

           ___
           \     ~   ~a    a   ~a  ~             1     _____    i*phase*(n-m)
            )  <psi |p > dO   <p |psi > =  +/-  --- * V m*n  * e
           /___    n  i    ii'  i'   n'          Z
            ii'                                   a

        """
        if self.allocated:
            raise RuntimeError('Already allocated!')

        # Fill in projector overlaps
        my_band_indices = self.bd.get_band_indices()
        my_atom_indices = np.argwhere(self.gd.comm.rank == self.rank_a).ravel()

        # Holm-Nielsen check:
        natoms = len(self.atoms)
        assert (self.gd.comm.sum(float(sum(my_atom_indices))) ==
                natoms * (natoms - 1) // 2)

        # Check that LFC agrees with us:
        self.assertEqual(len(my_atom_indices), len(self.pt.my_atom_indices))
        for a1, a2 in zip(my_atom_indices, self.pt.my_atom_indices):
            self.assertEqual(a1, a2)

        self.Qeff_a = {}
        self.P_ani = self.pt.dict(self.bd.mynbands)
        for a in my_atom_indices:
            ni = self.setups[a].ni
            # Fill P_ni: <p_i | psit_n > = beta_i * exp(i*phase*n) * sqrt(n)
            #
            #  |  ____                   |
            #  |  \        *    a        |      1
            #  |   )   beta   dO   beta  |  =  ----
            #  |  /___     i    ij     j |      Z
            #  |    ij                   |       a
            #
            # Substitution by linear transformation: beta_i ->  dO_ij alpha_j,
            # where we start out with some initial non-constant vector:
            alpha_i = np.exp(-np.arange(ni).astype(self.dtype)/ni)
            try:
                # Try Cholesky decomposition dO_ii = L_ii * L_ii^dag
                L_ii = np.linalg.cholesky(self.setups[a].dO_ii)
                alpha_i /= np.vdot(alpha_i, alpha_i)**0.5
                beta_i = np.linalg.solve(L_ii.T.conj(), alpha_i)
            except np.linalg.LinAlgError:
                # Eigenvector decomposition dO_ii = V_ii * W_ii * V_ii^dag
                W_i, V_ii = np.linalg.eigh(self.setups[a].dO_ii)
                alpha_i /= np.abs(np.vdot(alpha_i,
                                          np.dot(np.diag(W_i), alpha_i)))**0.5
                beta_i = np.linalg.solve(V_ii.T.conj(), alpha_i)

            # Normalize according to plus/minus charge
            beta_i /= self.Z_a[a]**0.5
            self.Qeff_a[a] = np.vdot(beta_i, np.dot(self.setups[a].dO_ii, \
                                                    beta_i)).real
            self.P_ani[a][:] = np.outer(self.gamma**my_band_indices \
                                        * my_band_indices**0.5, beta_i)

    def check_and_plot(self, A_nn, A0_nn, digits, keywords=''):
        # Construct fingerprint of input matrices for comparison
        fingerprint = np.array([md5_array(A_nn, numeric=True),
                                md5_array(A0_nn, numeric=True)])

        # Compare fingerprints across all processors
        fingerprints = np.empty((world.size, 2), np.int64)
        world.all_gather(fingerprint, fingerprints)
        if fingerprints.ptp(0).any():
            raise RuntimeError('Distributed matrices are not identical!')

        # If assertion fails, catch temporarily while plotting, then re-raise
        try:
            self.assertAlmostEqual(np.abs(A_nn-A0_nn).max(), 0, digits)
        except AssertionError:
            if world.rank == 0 and mpl is not None:
                from matplotlib.figure import Figure
                fig = Figure()
                ax = fig.add_axes([0.0, 0.1, 1.0, 0.83])
                ax.set_title(self.__class__.__name__)
                im = ax.imshow(np.abs(A_nn-A0_nn), interpolation='nearest')
                fig.colorbar(im)
                fig.text(0.5, 0.05, 'Keywords: ' + keywords, \
                    horizontalalignment='center', verticalalignment='top')

                from matplotlib.backends.backend_agg import FigureCanvasAgg
                img = 'ut_hsops_%s_%s.png' % (self.__class__.__name__, \
                    '_'.join(keywords.split(',')))
                FigureCanvasAgg(fig).print_figure(img.lower(), dpi=90)
            raise

    def get_optimal_number_of_blocks(self, blocking='fast'):
        """Estimate the optimal number of blocks for band parallelization.

        The number of blocks determines how many parallel send/receive
        operations are performed, as well as the added memory footprint
        of the required send/receive buffers.

        ``blocking``  ``nblocks``      Description
        ============  =============    ========================================
        'fast'        ``1``            Heavy on memory, more accurate and fast.
        'light'       ``mynbands``     Light on memory, less accurate and slow.
        'intdiv'      ``...``          First integer divisible value
        'nonintdiv'   ``...``          Some non-integer divisible cases
        """

        #if self.bd.comm.size == 1:
        #    return 1

        if blocking == 'fast':
            return 1
        elif blocking == 'light':
            return self.bd.mynbands
        elif blocking == 'intdiv':
            # Find first value of nblocks that leads to integer
            # divisible mybands / nblock. This is very like to be
            # 2 but coded here for the general case
            nblocks = 2
            while self.bd.mynbands % nblocks != 0:
                nblocks +=1
            return nblocks
        elif blocking == 'nonintdiv1':
            # Find first value of nblocks that leads to non-integer
            # divisible mynbands / nblock that is less than M
            nblocks = 2
            M = self.bd.mynbands // nblocks
            while self.bd.mynbands % nblocks < M:
                nblocks += 1
                M = self.bd.mynbands // nblocks
            return nblocks
        elif blocking == 'nonintdiv2':
            # Find first value of nblocks that leads to non-integer
            # divisible mynbands / nblock that is less than M
            nblocks = 2
            M = self.bd.mynbands // nblocks
            while self.bd.mynbands % nblocks > M:
                nblocks += 1
                M = self.mynbands // nblocks
            return nblocks
        else:
            nblocks = blocking
            assert self.bd.mynbands // nblocks > 0
            return nblocks

    # =================================

    def test_contents_wavefunction(self):
        # Integrate diagonal brakets of pseudo wavefunctions
        gpts_c = self.gd.get_size_of_global_array()

        intpsit_myn = self.bd.empty(dtype=self.dtype)
        for myn, psit_G in enumerate(self.psit_nG):
            n = self.bd.global_index(myn)
            intpsit_myn[myn] = np.vdot(psit_G, psit_G) * self.gd.dv
        self.gd.comm.sum(intpsit_myn)

        if memstats:
            self.mem_test = record_memory()

        my_band_indices = self.bd.get_band_indices()
        self.assertAlmostEqual(np.abs(intpsit_myn-my_band_indices).max(), 0, 9)

        intpsit_n = self.bd.collect(intpsit_myn, broadcast=True)
        self.assertAlmostEqual(np.abs(intpsit_n-np.arange(self.nbands)).max(), 0, 9)

    def test_contents_projection(self):
        # Distribute inverse effective charges to everybody in domain
        all_Qeff_a = np.empty(len(self.atoms), dtype=float)
        for a,rank in enumerate(self.rank_a):
            if rank == self.gd.comm.rank:
                Qeff = np.array([self.Qeff_a[a]])
            else:
                Qeff = np.empty(1, dtype=float)
            self.gd.comm.broadcast(Qeff, rank)
            all_Qeff_a[a] = Qeff

        # Check absolute values consistency of inverse effective charges
        self.assertAlmostEqual(np.abs(1./self.Z_a-np.abs(all_Qeff_a)).max(), 0, 9)

        # Check sum of inverse effective charges against total
        self.assertAlmostEqual(all_Qeff_a.sum(), self.Qtotal, 9)

        # Make sure that we all agree on inverse effective charges
        fingerprint = np.array([md5_array(all_Qeff_a, numeric=True)])
        all_fingerprints = np.empty(world.size, fingerprint.dtype)
        world.all_gather(fingerprint, all_fingerprints)
        if all_fingerprints.ptp(0).any():
            raise RuntimeError('Distributed eff. charges are not identical!')

    def test_overlaps_hermitian(self):
        # Set up Hermitian overlap operator:
        S = lambda x: x
        dS = lambda a, P_ni: np.dot(P_ni, self.setups[a].dO_ii)
        nblocks = self.get_optimal_number_of_blocks(self.blocking)
        overlap = MatrixOperator(self.ksl, nblocks, self.async, True)
        S_nn = overlap.calculate_matrix_elements(self.psit_nG, \
            self.P_ani, S, dS).T.copy() # transpose to get <psit_m|A|psit_n>
        tri2full(S_nn, 'U') # upper to lower...

        if self.bd.comm.rank == 0:
            self.gd.comm.broadcast(S_nn, 0)
        self.bd.comm.broadcast(S_nn, 0)

        if memstats:
            self.mem_test = record_memory()

        self.check_and_plot(S_nn, self.S0_nn, 9, 'overlaps,hermitian')

    def test_overlaps_nonhermitian(self):
        alpha = np.random.normal(size=1).astype(self.dtype)
        if self.dtype == complex:
            alpha += 1j*np.random.normal(size=1)
        world.broadcast(alpha, 0)

        # Set up non-Hermitian overlap operator:
        S = lambda x: alpha*x
        dS = lambda a, P_ni: np.dot(alpha*P_ni, self.setups[a].dO_ii)
        nblocks = self.get_optimal_number_of_blocks(self.blocking)
        overlap = MatrixOperator(self.ksl, nblocks, self.async, False)
        S_nn = overlap.calculate_matrix_elements(self.psit_nG, \
            self.P_ani, S, dS).T.copy() # transpose to get <psit_m|A|psit_n>

        if self.bd.comm.rank == 0:
            self.gd.comm.broadcast(S_nn, 0)
        self.bd.comm.broadcast(S_nn, 0)

        if memstats:
            self.mem_test = record_memory()

        self.check_and_plot(S_nn, alpha*self.S0_nn, 9, 'overlaps,nonhermitian')

    def test_trivial_cholesky(self):
        # Known starting point of SI_nn = <psit_m|S+alpha*I|psit_n>
        I_nn = np.eye(*self.S0_nn.shape)
        alpha = 1e-3 # shift eigenvalues away from zero
        SI_nn = self.S0_nn + alpha * I_nn

        # Try Cholesky decomposition SI_nn = L_nn * L_nn^dag
        L_nn = np.linalg.cholesky(SI_nn)
        # |psit_n> -> C_nn |psit_n> , C_nn^(-1) = L_nn^dag
        # <psit_m|SI|psit_n> -> <psit_m|C_nn^dag SI C_nn|psit_n> = diag(W_n)
        C_nn = np.linalg.inv(L_nn.T.conj())

        # Set up Hermitian overlap operator:
        S = lambda x: x
        dS = lambda a, P_ni: np.dot(P_ni, self.setups[a].dO_ii)
        nblocks = self.get_optimal_number_of_blocks(self.blocking)
        overlap = MatrixOperator(self.ksl, nblocks, self.async, True)
        self.psit_nG = overlap.matrix_multiply(C_nn.T.copy(), self.psit_nG, self.P_ani)
        D_nn = overlap.calculate_matrix_elements(self.psit_nG, \
            self.P_ani, S, dS).T.copy() # transpose to get <psit_m|A|psit_n>
        tri2full(D_nn, 'U') # upper to lower...

        if self.bd.comm.rank == 0:
            self.gd.comm.broadcast(D_nn, 0)
        self.bd.comm.broadcast(D_nn, 0)

        if memstats:
            self.mem_test = record_memory()

        # D_nn = C_nn^dag * S_nn * C_nn = I_nn - alpha * C_nn^dag * C_nn
        D0_nn = I_nn - alpha * np.dot(C_nn.T.conj(), C_nn)
        self.check_and_plot(D_nn, D0_nn, 6, 'trivial,cholesky') #XXX precision

    def test_trivial_diagonalize(self):
        # Known starting point of S_nn = <psit_m|S|psit_n>
        S_nn = self.S0_nn

        # Eigenvector decomposition S_nn = V_nn * W_nn * V_nn^dag
        # Utilize the fact that they are analytically known (cf. Maple)
        band_indices = np.arange(self.nbands)
        V_nn = np.eye(self.nbands).astype(self.dtype)
        if self.dtype == complex:
            V_nn[1:,1] = np.conj(self.gamma)**band_indices[1:] * band_indices[1:]**0.5
            V_nn[1,2:] = -self.gamma**band_indices[1:-1] * band_indices[2:]**0.5
        else:
            V_nn[2:,1] = band_indices[2:]**0.5
            V_nn[1,2:] = -band_indices[2:]**0.5

        W_n = np.zeros(self.nbands).astype(self.dtype)
        W_n[1] = (1. + self.Qtotal) * self.nbands * (self.nbands - 1) / 2.

        # Find the inverse basis
        Vinv_nn = np.linalg.inv(V_nn)

        # Test analytical eigenvectors for consistency against analytical S_nn
        D_nn = np.dot(Vinv_nn, np.dot(S_nn, V_nn))
        self.assertAlmostEqual(np.abs(D_nn.diagonal()-W_n).max(), 0, 8)
        self.assertAlmostEqual(np.abs(np.tril(D_nn, -1)).max(), 0, 4)
        self.assertAlmostEqual(np.abs(np.triu(D_nn, 1)).max(), 0, 4)
        del Vinv_nn, D_nn

        # Perform Gram Schmidt orthonormalization for diagonalization
        # |psit_n> -> C_nn |psit_n>, using orthonormalized basis Q_nn
        # <psit_m|S|psit_n> -> <psit_m|C_nn^dag S C_nn|psit_n> = diag(W_n)
        # using S_nn = V_nn * W_nn * V_nn^(-1) = Q_nn * W_nn * Q_nn^dag
        C_nn = V_nn.copy()
        gram_schmidt(C_nn)
        self.assertAlmostEqual(np.abs(np.dot(C_nn.T.conj(), C_nn) \
                                      - np.eye(self.nbands)).max(), 0, 6)

        # Set up Hermitian overlap operator:
        S = lambda x: x
        dS = lambda a, P_ni: np.dot(P_ni, self.setups[a].dO_ii)
        nblocks = self.get_optimal_number_of_blocks(self.blocking)
        overlap = MatrixOperator(self.ksl, nblocks, self.async, True)
        self.psit_nG = overlap.matrix_multiply(C_nn.T.copy(), self.psit_nG, self.P_ani)
        D_nn = overlap.calculate_matrix_elements(self.psit_nG, \
            self.P_ani, S, dS).T.copy() # transpose to get <psit_m|A|psit_n>
        tri2full(D_nn, 'U') # upper to lower...

        if self.bd.comm.rank == 0:
            self.gd.comm.broadcast(D_nn, 0)
        self.bd.comm.broadcast(D_nn, 0)

        if memstats:
            self.mem_test = record_memory()

        # D_nn = C_nn^dag * S_nn * C_nn = W_n since Q_nn^dag = Q_nn^(-1)
        D0_nn = np.dot(C_nn.T.conj(), np.dot(S_nn, C_nn))
        self.assertAlmostEqual(np.abs(D0_nn-np.diag(W_n)).max(), 0, 9)
        self.check_and_plot(D_nn, D0_nn, 9, 'trivial,diagonalize')

    def test_multiply_randomized(self):
        # Known starting point of S_nn = <psit_m|S|psit_n>
        S_nn = self.S0_nn

        if self.dtype == complex:
            C_nn = np.random.uniform(size=self.nbands**2) * \
                np.exp(1j*np.random.uniform(0,2*np.pi,size=self.nbands**2))
        else:
            C_nn = np.random.normal(size=self.nbands**2)
        C_nn = C_nn.reshape((self.nbands,self.nbands)) / np.linalg.norm(C_nn,2)
        world.broadcast(C_nn, 0)

        # Set up Hermitian overlap operator:
        S = lambda x: x
        dS = lambda a, P_ni: np.dot(P_ni, self.setups[a].dO_ii)
        nblocks = self.get_optimal_number_of_blocks(self.blocking)
        overlap = MatrixOperator(self.ksl, nblocks, self.async, True)
        self.psit_nG = overlap.matrix_multiply(C_nn.T.copy(), self.psit_nG, self.P_ani)
        D_nn = overlap.calculate_matrix_elements(self.psit_nG, \
            self.P_ani, S, dS).T.copy() # transpose to get <psit_m|A|psit_n>
        tri2full(D_nn, 'U') # upper to lower...

        if self.bd.comm.rank == 0:
            self.gd.comm.broadcast(D_nn, 0)
        self.bd.comm.broadcast(D_nn, 0)

        if memstats:
            self.mem_test = record_memory()

        # D_nn = C_nn^dag * S_nn * C_nn
        D0_nn = np.dot(C_nn.T.conj(), np.dot(S_nn, C_nn))
        self.check_and_plot(D_nn, D0_nn, 9, 'multiply,randomized')

    def test_multiply_nonhermitian(self):
        alpha = np.random.normal(size=1).astype(self.dtype)
        if self.dtype == complex:
            alpha += 1j*np.random.normal(size=1)
        world.broadcast(alpha, 0)

        # Known starting point of S_nn = <psit_m|S|psit_n>
        S_nn = alpha*self.S0_nn

        if self.dtype == complex:
            C_nn = np.random.uniform(size=self.nbands**2) * \
                np.exp(1j*np.random.uniform(0,2*np.pi,size=self.nbands**2))
        else:
            C_nn = np.random.normal(size=self.nbands**2)
        C_nn = C_nn.reshape((self.nbands,self.nbands)) / np.linalg.norm(C_nn,2)
        world.broadcast(C_nn, 0)

        # Set up non-Hermitian overlap operator:
        S = lambda x: alpha*x
        dS = lambda a, P_ni: np.dot(alpha*P_ni, self.setups[a].dO_ii)
        nblocks = self.get_optimal_number_of_blocks(self.blocking)
        overlap = MatrixOperator(self.ksl, nblocks, self.async, False)
        self.psit_nG = overlap.matrix_multiply(C_nn.T.copy(), self.psit_nG, self.P_ani)
        D_nn = overlap.calculate_matrix_elements(self.psit_nG, \
            self.P_ani, S, dS).T.copy() # transpose to get <psit_m|A|psit_n>

        if self.bd.comm.rank == 0:
            self.gd.comm.broadcast(D_nn, 0)
        self.bd.comm.broadcast(D_nn, 0)

        if memstats:
            self.mem_test = record_memory()

        # D_nn = C_nn^dag * S_nn * C_nn
        D0_nn = np.dot(C_nn.T.conj(), np.dot(S_nn, C_nn))
        self.check_and_plot(D_nn, D0_nn, 9, 'multiply,nonhermitian')


# -------------------------------------------------------------------

def UTConstantWavefunctionFactory(dtype, parstride_bands, blocking, async):
    sep = '_'
    classname = 'UTConstantWavefunctionSetup' \
    + sep + {float:'Float', complex:'Complex'}[dtype] \
    + sep + {False:'Blocked', True:'Strided'}[parstride_bands] \
    + sep + {'fast':'Fast', 'light':'Light',
             'intdiv':'Intdiv', 'nonintdiv1':'Nonintdiv1',
             'nonintdiv2':'Nonintdiv2'}[blocking] \
    + sep + {False:'Synchronous', True:'Asynchronous'}[async]
    class MetaPrototype(UTConstantWavefunctionSetup, object):
        __doc__ = UTConstantWavefunctionSetup.__doc__
        dtype = dtype
        parstride_bands = parstride_bands
        blocking = blocking
        async = async
    MetaPrototype.__name__ = classname
    return MetaPrototype

# -------------------------------------------------------------------

if __name__ in ['__main__', '__builtin__']:
    # We may have been imported by test.py, if so we should redirect to logfile
    if __name__ == '__builtin__':
        testrunner = CustomTextTestRunner('ut_hsops.log', verbosity=2)
    else:
        stream = (world.rank == 0) and sys.stdout or devnull
        testrunner = TextTestRunner(stream=stream, verbosity=2)

    parinfo = []
    # Initial Verification only tests case : dtype = float
    for test in [UTBandParallelSetup_Blocked, UTBandParallelSetup_Strided]:
        info = ['', test.__name__, test.__doc__.strip('\n'), '']
        testsuite = initialTestLoader.loadTestsFromTestCase(test)
        list(map(testrunner.stream.writeln, info))
        testresult = testrunner.run(testsuite)
        assert testresult.wasSuccessful(), 'Initial verification failed!'
        parinfo.extend(['    Parallelization options: %s' % tci._parinfo for \
                        tci in testsuite._tests if hasattr(tci, '_parinfo')])
    parinfo = np.unique(np.sort(parinfo)).tolist()

    testcases = []
    for dtype in [float, complex]:
        for parstride_bands in [False, True]:
            for blocking in ['fast', 'light', 'intdiv',
                             'nonintdiv1', 'nonintdiv2']:
                for async in [False, True]:
                    testcases.append(UTConstantWavefunctionFactory(dtype, \
                        parstride_bands, blocking, async))

    for test in testcases:
        info = ['', test.__name__, test.__doc__.strip('\n')] + parinfo + ['']
        testsuite = defaultTestLoader.loadTestsFromTestCase(test)
        list(map(testrunner.stream.writeln, info))
        testresult = testrunner.run(testsuite)
        # Provide feedback on failed tests if imported by test.py
        if __name__ == '__builtin__' and not testresult.wasSuccessful():
            raise SystemExit('Test failed. Check ut_hsops.log for details.')




from __future__ import print_function
import numpy as np
from ase.lattice import bulk
from ase.dft.kpoints import monkhorst_pack
from ase.units import Bohr

from gpaw import GPAW
from gpaw.response.chi import CHI
from gpaw.response.chi0 import Chi0
from gpaw.mpi import serial_comm


omega = np.array([0, 1.0, 2.0])
for k in [2, 3]:
    q_c = [0, 0, 1.0 / k]
    for gamma in [False, True]:
        if k == 3 and gamma:
            continue
        kpts = monkhorst_pack((k, k, k))
        if gamma:
            kpts += 0.5 / k
        for center in [False, True]:
            a = bulk('Si', 'diamond')
            if center:
                a.center()
            for sym in [False, True]:
                name = 'si.k%d.g%d.c%d.s%d' % (k, gamma, center, bool(sym))
                print(name)
                if 1:
                    calc = a.calc = GPAW(
                        kpts=kpts,
                        eigensolver='rmm-diis',
                        symmetry={'point_group': sym},
                        mode='pw',
                        width=0.001,
                        txt=name + '.txt')
                    e = a.get_potential_energy()
                    #calc.diagonalize_full_hamiltonian(nbands=100)
                    calc.write(name, 'all')
                    
                calc = GPAW(name, txt=None, communicator=serial_comm)

                chiold = CHI(calc, w=omega, q=q_c, ecut=100,
                             hilbert_trans=False, xc='RPA',
                             G_plus_q=True, txt=name + '.logold')
                chiold.initialize()
                chiold.calculate()
                chi0old_wGG = chiold.chi0_wGG

                chi = Chi0(calc, omega, hilbert=False,
                           ecut=100, txt=name + '.log')
                pd, chi0_wGG, _, _ = chi.calculate(q_c)
                
                assert abs(chi0_wGG - chi0old_wGG).max() < 1e-8
                
                if not sym and not center:
                    chi00_w = chi0_wGG[:, 0, 0]
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG[:, 0, 0] - chi00_w).max() < 3e-5
                    #print abs(chi0_wGG[:, 0, 0] - chi00_w).max()
                    
                if not sym:
                    chi00_wGG = chi0_wGG
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG - chi00_wGG).max() < 2e-5
                    #print abs(chi0_wGG - chi00_wGG).max()

                q0_c = [0, 1e-7, 1e-7]
                q0_v = np.dot(q0_c, a.get_reciprocal_cell() * 2 * np.pi) * Bohr
                q0 = (q0_v**2).sum()**0.5
                
                chiold = CHI(calc, w=omega, q=q0_c, ecut=100,
                             hilbert_trans=False, xc='RPA', optical_limit=True,
                             G_plus_q=True, txt=name + '.logold0')
                chiold.initialize()
                chiold.calculate()
                chi0old_wGG = chiold.chi0_wGG
                chi0old_wGG[:, 0] /= q0
                chi0old_wGG[:, :, 0] /= q0
                
                pd, chi0_wGG, _, _ = chi.calculate([0, 0, 0])
                
                assert abs(chi0_wGG - chi0old_wGG).max() < 0.003
                assert abs(chi0_wGG - chi0old_wGG)[:, 1:, 1:].max() < 1e-8

                if not sym and not center:
                    chi000_w = chi0_wGG[:, 0, 0]
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG[:, 0, 0] - chi000_w).max() < 0.001
                    
                if not sym:
                    chi000_wGG = chi0_wGG
                elif -1 not in calc.wfs.kd.bz2bz_ks:
                    assert abs(chi0_wGG - chi000_wGG).max() < 0.001



# -*- coding: utf-8
# In an attempt to appease epydoc and still have readable docstrings,
# the vertical bar | is represented by u'\u2758' in this module.
"""This module defines Coulomb and XC kernels for the response model.
"""

import numpy as np
#from math import sqrt, pi, sin, cos, exp
from gpaw.utilities.blas import gemmdot
from gpaw.xc import XC
from gpaw.sphere.lebedev import weight_n, R_nv
from gpaw.mpi import world, rank, size
from ase.dft.kpoints import monkhorst_pack


def v3D_Coulomb(qG):
    """Coulomb Potential in the 3D Periodic Case.
    
    Periodic calculation, no cutoff.

    ::
    
      v3D = 4 pi / |q+G|^2
    """
    
    v_q = 1. / (qG**2).sum(axis=1)
    return v_q


def v2D_Coulomb(qG, N_p, N_np, R):
    """Coulomb Potential in the 2D Periodic Case.

    Slab/Surface/Layer calculation.
    Cutoff in non-periodic N_np direction.
    No cutoff in periodic N_p directions.

    ::

      v2D = 4 pi/|q+G|^2 * [1 + exp(-|G_p|R)*[(G_n/|G_p|)*sin(G_n R)
                                                        - cos(G_n R)]
    """

    G_nR = qG[:, N_np[0]] * R
    G_pR = (qG[:, N_p[0]]**2 + qG[:, N_p[1]]**2)**0.5 * R

    v_q = 1. / (qG**2).sum(axis=1)
    v_q *= 1. + np.exp(-G_pR) * ((G_nR / G_pR) * np.sin(G_nR)
                                 - np.cos(G_nR))
    return v_q.astype(complex)


def v1D_Coulomb(qG, N_p, N_np, R):
    """Coulomb Potential in the 1D Periodic Case.
    
    Nanotube/Nanowire/Atomic Chain calculation.
    Cutoff in non-periodic N_np directions.
    No cutoff in periodic N_p direction.

    ::
    
      v1D = 4 pi/|q+G|^2 * [1 + |G_n|R J_1(|G_n|R) K_0(G_p R)
                              - G_p R J_0(|G_n| R) K_1(G_p R)]
    """

    from scipy.special import j1, k0, j0, k1

    G_nR = (qG[:, N_np[0]]**2 + qG[:, N_np[1]]**2)**0.5 * R
    G_pR = abs(qG[:, N_p[0]]) * R
    v_q = 1. / (qG**2).sum(axis=1)
    v_q *= (1. + G_nR * j1(G_nR) * k0(G_pR)
            - G_pR * j0(G_nR) * k1(G_pR))
    return v_q


def v0D_Coulomb(qG, R):
    """Coulomb Potential in the 0D Non-Periodic Case

    Isolated System/Molecule calculation.
    Spherical cutoff in all directions.

    ::

      v0D = 4 pi/|G|^2 * [1 - cos(|G|R)]
    """

    qGR = ((qG.sum(axis=1))**2)**0.5 * R
    v_q = 1. / (qG**2).sum(axis=1)
    v_q *= 1. - np.cos(qGR)
    return v_q


class BaseCoulombKernel:
    """This is an abstract class for calculating the Coulomb kernel.

    """
    def __init__(self, vcut):
        self.vcut = vcut

    def v_Coulomb(self, qG):
        """Returns the Coulomb potential in k-space.
        """
        pass

    def calculate_Kc(self,
                     q_c,
                     Gvec_Gc,
                     bcell_cv,
                     integrate_gamma=False,
                     N_k=None,
                     symmetric=True):
        """Symmetric Coulomb kernel.
        """
        pass

    def calculate_Kc_q(self,
                       bcell_cv,
                       N_k,
                       q_qc=None,
                       Gvec_c=None,
                       N=100):
        """What does this do? XXX
        """
        pass


class CoulombKernel3D(BaseCoulombKernel):
    """Class for calculating the Coulomb kernel for a 3D calculation.

    vcut = [False, False, False].
    """
    def v_Coulomb(self, qG):
        """Coulomb Potential in the 3D Periodic Case.
        Periodic calculation, no cutoff.
        """
        return v3D_Coulomb(qG)


class CoulombKernel2D(BaseCoulombKernel):
    """Class for calculating the Coulomb kernel for a 2D calculation.
    
    Slab/Surface/Layer calculation.
    vcut = [False, False, True].
    Cutoff in non-periodic N_np direction.
    No cutoff in periodic N_p directions.
    R = 1/2 max cell[N_np].
    """
    def __init__(self, vcut, cell):
        # Periodic Directions
        self.N_p = vcut.argsort()[:2]
        # Non-Periodic Direction
        self.N_np = vcut.argsort()[-1:]
        # 2D Radial Cutoff is one half the cell dimension in Bohr
        self.R = cell[self.N_np, self.N_np] / 2.
        BaseCoulombKernel.__init__(self, vcut)

    def v_Coulomb(self, qG):
        """Coulomb Potential in the 2D Periodic Case.

        Slab/Surface/Layer calculation.
        Cutoff in non-periodic N_np direction.
        No cutoff in periodic N_p directions.
        """
        
        return v2D_Coulomb(qG, self.N_p, self.N_np, self.R)


class CoulombKernel1D(BaseCoulombKernel):
    """Class for calculating the Coulomb kernel for a 1D calculation.
 
    Nanotube/Nanowire/Atomic Chain calculation.
    vcut = [False, True, True].
    Cutoff in non-periodic N_np directions.
    No cutoff in periodic N_p direction.
    R = 1/2 max cell[N_np].
    """
    def __init__(self, vcut, cell):
        from scipy.special import j1, k0, j0, k1
        # Periodic Direction
        self.N_p = vcut.argsort()[:1]
        # Non-Periodic Directions
        self.N_np = vcut.argsort()[-2:]
        # 2D Radial Cutoff is one half the cell dimension in Bohr
        self.R = max(cell[self.N_np[0], self.N_np[0]],
                     cell[self.N_np[1], self.N_np[1]]) / 2.
        self.j1 = j1
        self.k0 = k0
        self.j0 = j0
        self.k1 = k1
        BaseCoulombKernel.__init__(self, vcut)

    def v_Coulomb(self, qG):
        """Coulomb Potential in the 1D Periodic Case.

        Nanotube/Nanowire/Atomic Chain calculation.
        Cutoff in non-periodic N_np directions.
        No cutoff in periodic N_p direction.
        """

        return v1D_Coulomb(qG, self.N_p, self.N_np, self.R)


class CoulombKernel0D(BaseCoulombKernel):
    """Class for calculating the Coulomb kernel for a 0D calculation.
    
    Isolated System/Molecule calculation.
    vcut = [True, True, True].
    Spherical cutoff in all directions.
    R = 1/2 max cell.
    """
    def __init__(self, vcut, cell):
        self.R = max(np.diag(cell)) / 2.
        BaseCoulombKernel.__init__(self, vcut)

    def v_Coulomb(self, qG):
        """Coulomb Potential in the 0D Non-Periodic Case

        Isolated System/Molecule calculation.
        Spherical cutoff in all directions.
        """

        return v0D_Coulomb(qG, self.R)


class CoulombKernel(BaseCoulombKernel):
    """Class for calculating the Coulomb kernel.

    vcut = None (Default).
    Applies Coulomb cutoff in non-periodic directions.
    vcut = '3D' => [False, False, False].
    vcut = '2D' => [False, False, True].
    vcut = '1D' => [False, True, True].
    vcut = '0D' => [True, True, True].
    vcut is a numpy arry of type bool of length 3.
    
    """
    def __init__(self, vcut, pbc, cell):
        if vcut is None:
            vcut = ~pbc
        elif isinstance(vcut, str):
            # Assume vcut = 'nD', where n is number of periodic directions
            # Should be deprecated
            n_p = int(vcut[0])
            vcut = np.ones(3, bool)
            vcut[:n_p] = False

        # 3D periodic case
        if vcut.sum() == 0:
            self.impl = CoulombKernel3D(vcut)
        # 2D periodic case
        elif vcut.sum() == 1:
            self.impl = CoulombKernel2D(vcut, cell)
        # 1D periodic case
        elif vcut.sum() == 2:
            self.impl = CoulombKernel1D(vcut, cell)
        # 0D periodic case
        elif vcut.sum() == 3:
            self.impl = CoulombKernel0D(vcut, cell)
        
        BaseCoulombKernel.__init__(self, vcut)

    def v_Coulomb(self, qG):
        """Returns vcut dependent Coulomb potential.
        
        Uses v_Coulomb as implemented in self.impl
        """
        
        return self.impl.v_Coulomb(qG)

    def calculate_Kc(self,
                     q_c,
                     Gvec_Gc,
                     bcell_cv,
                     integrate_gamma=False,
                     N_k=None,
                     symmetric=True):

        """Symmetric Coulomb kernel"""

        if integrate_gamma:
            assert N_k is not None
            if (q_c == 0).all():
                # Only to avoid dividing by zero below!
                # The term is handled separately later
                q_c = [1.e-12, 0., 0.]

        qG_c = np.zeros((len(Gvec_Gc), 3))
        qG_c[:, 0] = Gvec_Gc[:, 0] + q_c[0]
        qG_c[:, 1] = Gvec_Gc[:, 1] + q_c[1]
        qG_c[:, 2] = Gvec_Gc[:, 2] + q_c[2]

        qG = np.dot(qG_c, bcell_cv)

        # Calculate coulomb kernel
        Kc_G = self.v_Coulomb(qG)

        if integrate_gamma and np.abs(q_c).sum() < 1e-8:
            bvol = np.abs(np.linalg.det(bcell_cv)) / (N_k[0] * N_k[1] * N_k[2])
            R_q0 = (3. * bvol / (4. * np.pi))**(1. / 3.)
            Kc_G[0] = 4. * np.pi * R_q0 / bvol

        if symmetric:
            Kc_G = np.array(Kc_G, np.complex)**0.5
            Kc_G = np.outer(Kc_G.conj(), Kc_G)
            return 4. * np.pi * Kc_G
        else:
            return 4. * np.pi * (Kc_G * np.ones([len(Kc_G), len(Kc_G)])).T

    def calculate_Kc_q(self,
                       bcell_cv,
                       N_k,
                       q_qc=None,
                       Gvec_c=None,
                       N=100):

        if q_qc is None:
            q_qc = np.array([[1.e-12, 0., 0.]])
        if Gvec_c is None:
            Gvec_c = np.array([0, 0, 0])

        bcell = bcell_cv.copy()
        bcell[0] /= N_k[0]
        bcell[1] /= N_k[1]
        bcell[2] /= N_k[2]
        bvol = np.abs(np.linalg.det(bcell))

        gamma = np.where(np.sum(abs(q_qc), axis=1) < 1e-5)[0][0]
        if (Gvec_c == 0).all():
            q_qc = q_qc.copy()
            # Just to avoid zero division
            q_qc[gamma] = [0.001, 0., 0.]

        q_qc[:, 0] += Gvec_c[0]
        q_qc[:, 1] += Gvec_c[1]
        q_qc[:, 2] += Gvec_c[2]
        qG = np.dot(q_qc, bcell_cv)

        K0_q = self.v_Coulomb(qG)
        if (Gvec_c == 0).all():
            R_q0 = (3. * bvol / (4. * np.pi))**(1. / 3.)
            K0_q[gamma] = 4. * np.pi * R_q0 / bvol

        bvol = np.abs(np.linalg.det(bcell))
        # Integrated Coulomb kernel
        qt_qc = monkhorst_pack((N, N, N))
        qt_qcv = np.dot(qt_qc, bcell)
        dv = bvol / len(qt_qcv)
        K_q = np.zeros(len(q_qc), complex)

        for i in range(len(q_qc)):
            q = qt_qcv.copy() + qG[i]
            v_q = v3D_Coulomb(q)
            K_q[i] = np.sum(v_q) * dv / bvol

        return 4. * np.pi * K_q, 4. * np.pi * K0_q


def calculate_Kc(q_c,
                 Gvec_Gc,
                 acell_cv,
                 bcell_cv,
                 pbc,
                 vcut=None,
                 integrate_gamma=False,
                 N_k=None,
                 symmetric=True):

    """Symmetric Coulomb kernel"""

    if integrate_gamma:
        assert N_k is not None
        if (q_c == 0).all():
            # Only to avoid dividing by zero below!
            # The term is handled separately later
            q_c = [1.e-12, 0., 0.]

    #G_p = np.array(pbc, float)
    #G_n = np.array([1,1,1]) - G_p
    if vcut is None:
        vcut = '3D'
    G_p = np.array(pbc, bool).argsort()[-int(vcut[0]):]
    G_n = np.array(pbc, bool).argsort()[:int(vcut[0])]

    if vcut is None or vcut == '3D':
        pass
    elif vcut == '2D':
        # R is half the length of cell in non-periodic direction
        if pbc.all():  # G_n.sum() < 1e-8: # default dir is z
            G_n = np.array([2])  # np.array([0,0,1])
            G_p = np.array([0, 1])  # np.array([1,1,0])
        acell_n = acell_cv[G_n, G_n]
        R = max(acell_n) / 2.
    elif vcut == '1D':
        # R is the radius of the cylinder containing the cell.
        if pbc.all():  # G_n.sum() < 1e-8:
            raise ValueError('Check boundary condition ! ')
        acell_n = acell_cv
        R = max(acell_n[G_n[0], G_n[0]],
                acell_n[G_n[1], G_n[1]]) / 2.
    elif vcut == '0D':
        # R is the minimum radius of a sphere containing the cell.
        acell_n = acell_cv
        R = min(acell_n[0, 0],
                acell_n[1, 1],
                acell_n[2, 2]) / 2.
    else:
        NotImplemented

    qG_c = np.zeros((len(Gvec_Gc), 3))
    qG_c[:, 0] = Gvec_Gc[:, 0] + q_c[0]
    qG_c[:, 1] = Gvec_Gc[:, 1] + q_c[1]
    qG_c[:, 2] = Gvec_Gc[:, 2] + q_c[2]

    qG = np.dot(qG_c, bcell_cv)

    # Calculate Coulomb kernel
    if vcut is None or vcut == '3D':
        Kc_G = v3D_Coulomb(qG)
    elif vcut == '2D':
        Kc_G = v2D_Coulomb(qG, G_p, G_n, R)
    elif vcut == '1D':
        Kc_G = v1D_Coulomb(qG, G_p, G_n, R)
    elif vcut == '0D':
        Kc_G = v0D_Coulomb(qG, R)
    else:
        NotImplemented

    if integrate_gamma and np.abs(q_c).sum() < 1e-8:
        bvol = np.abs(np.linalg.det(bcell_cv)) / (N_k[0] * N_k[1] * N_k[2])
        R_q0 = (3. * bvol / (4. * np.pi))**(1. / 3.)
        Kc_G[0] = 4. * np.pi * R_q0 / bvol

    if symmetric:
        #print "Kc_G"
        #print Kc_G
        Kc_G = np.array(Kc_G, np.complex)**0.5
        #print "Kc_G^0.5"
        #print Kc_GC
        #print "Outer Product"
        Kc_G = np.outer(Kc_G.conj(), Kc_G)
        #print Kc_G
        return 4. * np.pi * Kc_G  # np.outer(Kc_G.conj(), Kc_G)
    else:
        return 4. * np.pi * (Kc_G * np.ones([len(Kc_G), len(Kc_G)])).T


def calculate_Kxc(gd,
                  nt_sG,
                  npw,
                  Gvec_Gc,
                  nG,
                  vol,
                  bcell_cv,
                  R_av,
                  setups,
                  D_asp,
                  functional='ALDA',
                  density_cut=None):
    """ALDA kernel"""

    # The soft part
    #assert np.abs(nt_sG[0].shape - nG).sum() == 0
    if functional == 'ALDA_X':
        x_only = True
        A_x = -3. / 4. * (3. / np.pi)**(1. / 3.)
        nspins = len(nt_sG)
        assert nspins in [1, 2]
        fxc_sg = nspins**(1. / 3.) * 4. / 9. * A_x * nt_sG**(-2. / 3.)
    else:
        assert len(nt_sG) == 1
        x_only = False
        fxc_sg = np.zeros_like(nt_sG)
        xc = XC(functional[1:])
        xc.calculate_fxc(gd, nt_sG, fxc_sg)

    if density_cut is not None:
        fxc_sg[np.where(nt_sG * len(nt_sG) < density_cut)] = 0.0

    # FFT fxc(r)
    nG0 = nG[0] * nG[1] * nG[2]
    tmp_sg = [np.fft.fftn(fxc_sg[s]) * vol / nG0 for s in range(len(nt_sG))]

    r_vg = gd.get_grid_point_coordinates()
    Kxc_sGG = np.zeros((len(fxc_sg), npw, npw), dtype=complex)
    for s in range(len(fxc_sg)):
        for iG in range(npw):
            for jG in range(npw):
                dG_c = Gvec_Gc[iG] - Gvec_Gc[jG]
                if (nG / 2 - np.abs(dG_c) > 0).all():
                    index = (dG_c + nG) % nG
                    Kxc_sGG[s, iG, jG] = tmp_sg[s][index[0],
                                                   index[1],
                                                   index[2]]
                else:  # not in the fft index
                    dG_v = np.dot(dG_c, bcell_cv)
                    dGr_g = gemmdot(dG_v, r_vg, beta=0.0)
                    Kxc_sGG[s, iG, jG] = gd.integrate(np.exp(-1j * dGr_g)
                                                      * fxc_sg[s])

    # The PAW part
    KxcPAW_sGG = np.zeros_like(Kxc_sGG)
    dG_GGv = np.zeros((npw, npw, 3))
    for iG in range(npw):
        for jG in range(npw):
            dG_c = Gvec_Gc[iG] - Gvec_Gc[jG]
            dG_GGv[iG, jG] = np.dot(dG_c, bcell_cv)

    for a, setup in enumerate(setups):
        if rank == a % size:
            rgd = setup.xc_correction.rgd
            n_qg = setup.xc_correction.n_qg
            nt_qg = setup.xc_correction.nt_qg
            nc_g = setup.xc_correction.nc_g
            nct_g = setup.xc_correction.nct_g
            Y_nL = setup.xc_correction.Y_nL
            dv_g = rgd.dv_g

            D_sp = D_asp[a]
            B_pqL = setup.xc_correction.B_pqL
            D_sLq = np.inner(D_sp, B_pqL.T)
            nspins = len(D_sp)

            f_sg = rgd.empty(nspins)
            ft_sg = rgd.empty(nspins)

            n_sLg = np.dot(D_sLq, n_qg)
            nt_sLg = np.dot(D_sLq, nt_qg)

            # Add core density
            n_sLg[:, 0] += np.sqrt(4. * np.pi) / nspins * nc_g
            nt_sLg[:, 0] += np.sqrt(4. * np.pi) / nspins * nct_g

            coefatoms_GG = np.exp(-1j * np.inner(dG_GGv, R_av[a]))
            for n, Y_L in enumerate(Y_nL):
                w = weight_n[n]
                f_sg[:] = 0.0
                n_sg = np.dot(Y_L, n_sLg)
                if x_only:
                    f_sg = nspins * (4 / 9.) * A_x * (nspins * n_sg)**(-2 / 3.)
                else:
                    xc.calculate_fxc(rgd, n_sg, f_sg)

                ft_sg[:] = 0.0
                nt_sg = np.dot(Y_L, nt_sLg)
                if x_only:
                    ft_sg = nspins * (4 / 9.) * (A_x
                                                 * (nspins * nt_sg)**(-2 / 3.))
                else:
                    xc.calculate_fxc(rgd, nt_sg, ft_sg)
                for i in range(len(rgd.r_g)):
                    coef_GG = np.exp(-1j * np.inner(dG_GGv, R_nv[n])
                                     * rgd.r_g[i])
                    for s in range(len(f_sg)):
                        KxcPAW_sGG[s] += w * np.dot(coef_GG,
                                                    (f_sg[s, i] -
                                                     ft_sg[s, i])
                                                    * dv_g[i]) * coefatoms_GG

    world.sum(KxcPAW_sGG)
    Kxc_sGG += KxcPAW_sGG

    return Kxc_sGG / vol


def calculate_Kc_q(acell_cv,
                   bcell_cv,
                   pbc,
                   N_k,
                   vcut=None,
                   integrated=True,
                   q_qc=np.array([[1.e-12, 0., 0.]]),
                   Gvec_c=np.array([0, 0, 0]),
                   N=100):
    # get cutoff parameters
    #G_p = np.array(pbc, int)
    # Normal Direction
    #G_n = np.array([1,1,1]) - G_p
    if vcut is None:
        vcut = '3D'
    G_p = np.array(pbc, bool).argsort()[-int(vcut[0]):]
    G_n = np.array(pbc, bool).argsort()[:int(vcut[0])]

    if vcut is None or vcut == '3D':
        pass
    elif vcut == '2D':
        if pbc.all():  # G_n.sum() < 1e-8: # default dir is z
            G_n = np.array([2])
            G_p = np.array([0, 1])
        acell_n = acell_cv[G_n, G_n]
        R = max(acell_n) / 2.
    elif vcut == '1D':
        # R is the radius of the cylinder containing the cell.
        if pbc.all():  # G_n.sum() < 1e-8:
            raise ValueError('Check boundary condition ! ')
        acell_n = acell_cv
        R = max(acell_n[G_n[0], G_n[0]], acell_n[G_n[1], G_n[1]]) / 2.
    elif vcut == '0D':
        # R is the minimum radius of a sphere containing the cell.
        acell_n = acell_cv
        R = min(acell_n[0, 0], acell_n[1, 1], acell_n[2, 2]) / 2.
    else:
        NotImplemented

    bcell = bcell_cv.copy()
    bcell[0] /= N_k[0]
    bcell[1] /= N_k[1]
    bcell[2] /= N_k[2]
    bvol = np.abs(np.linalg.det(bcell))

    gamma = np.where(np.sum(abs(q_qc), axis=1) < 1e-5)[0][0]

    if (Gvec_c[G_p] == 0).all():
        q_qc[gamma][G_p[0]] = 1.e-12  # Just to avoid zero division

    q_qc[:, 0] += Gvec_c[0]
    q_qc[:, 1] += Gvec_c[1]
    q_qc[:, 2] += Gvec_c[2]
    qG = np.dot(q_qc, bcell_cv)

    if vcut is None or vcut == '3D':
        K0_q = v3D_Coulomb(qG)
        if (Gvec_c == 0).all():
            R_q0 = (3. * bvol / (4. * np.pi))**(1. / 3.)
            K0_q[gamma] = 4. * np.pi * R_q0 / bvol
    elif vcut == '2D':
        K0_q = v2D_Coulomb(qG, G_p, G_n, R)
        if (Gvec_c == 0).all():
            R_q0 = (3 * bvol / (4. * np.pi))**(1. / 3.)
            K0_q[gamma] = (4. * np.pi * R_q0 / bvol)
    elif vcut == '1D':
        K0_q = v1D_Coulomb(qG, G_p, G_n, R)
    elif vcut == '0D':
        K0_q = v0D_Coulomb(qG, R)
    else:
        NotImplemented

    bvol = np.abs(np.linalg.det(bcell))
    # Integrated Coulomb kernel
    qt_qc = monkhorst_pack((N, N, N))
    qt_qcv = np.dot(qt_qc, bcell)
    dv = bvol / len(qt_qcv)
    K_q = np.zeros(len(q_qc), complex)

    for i in range(len(q_qc)):
        q = qt_qcv.copy() + qG[i]
        if vcut is None or vcut == '3D':
            v_q = v3D_Coulomb(q)
        elif vcut == '2D':
            v_q = v2D_Coulomb(q, G_p, G_n, R)
        elif vcut == '1D':
            v_q = v1D_Coulomb(q, G_p, G_n, R)
        elif vcut == '0D':
            v_q = v0D_Coulomb(q, R)
        else:
            NotImplemented
        K_q[i] = np.sum(v_q) * dv / bvol

    return 4. * np.pi * K_q, 4. * np.pi * K0_q



"""User provided customizations.

Here one changes the default arguments for compiling _gpaw.so (serial)
and gpaw-python (parallel).

Here are all the lists that can be modified:
    
* libraries
* library_dirs
* include_dirs
* extra_link_args
* extra_compile_args
* runtime_library_dirs
* extra_objects
* define_macros
* mpi_libraries
* mpi_library_dirs
* mpi_include_dirs
* mpi_runtime_library_dirs
* mpi_define_macros

To override use the form:
    
    libraries = ['somelib', 'otherlib']

To append use the form

    libraries += ['somelib', 'otherlib']
"""

# compiler = 'gcc'
# mpicompiler = 'mpicc'  # use None if you don't want to build a gpaw-python
# mpilinker = 'mpicc'
# platform_id = ''
# scalapack = False
# hdf5 = False

# Use ScaLAPACK:
# Warning! At least scalapack 2.0.1 is required!
# See https://trac.fysik.dtu.dk/projects/gpaw/ticket/230
if scalapack:
    libraries += ['scalapack-openmpi',
                  'blacsCinit-openmpi',
                  'blacs-openmpi']
    define_macros += [('GPAW_NO_UNDERSCORE_CBLACS', '1')]
    define_macros += [('GPAW_NO_UNDERSCORE_CSCALAPACK', '1')]

# LibXC:
# In order to link libxc installed in a non-standard location
# (e.g.: configure --prefix=/home/user/libxc-2.0.1-1), use:

# - static linking:
if 0:
    include_dirs += ['/home/user/libxc-2.0.1-1/include']
    extra_link_args += ['/home/user/libxc-2.0.1-1/lib/libxc.a']
    if 'xc' in libraries:
        libraries.remove('xc')
        
# - dynamic linking (requires rpath or setting LD_LIBRARY_PATH at runtime):
if 0:
    include_dirs += ['/home/user/libxc-2.0.1-1/include']
    library_dirs += ['/home/user/libxc-2.0.1-1/lib']
    # You can use rpath to avoid changing LD_LIBRARY_PATH:
    # extra_link_args += ['-Wl,-rpath=/home/user/libxc-2.0.1-1/lib']
    if 'xc' not in libraries:
        libraries.append('xc')


# libvdwxc:
if 0:
    libvdwxc = True
    path = '/home/user/libvdwxc'
    extra_link_args += ['-Wl,-rpath=%s/lib' % path]
    library_dirs += ['%s/lib' % path]
    include_dirs += ['%s/include' % path]
    libraries += ['vdwxc']


# Build MPI-interface into _gpaw.so:
if 0:
    compiler = 'mpicc'
    define_macros += [('PARALLEL', '1')]
    mpicompiler = None



from __future__ import print_function
import sys
from time import ctime, time
import numpy as np

from ase.parallel import paropen
from ase.units import Hartree, Bohr
from ase.utils import devnull

from gpaw import GPAW
from gpaw.xc import XC
from gpaw.xc.libxc import LibXC
from gpaw.response.df0 import DF
from gpaw.utilities.blas import gemmdot, axpy
from gpaw.kpt_descriptor import KPointDescriptor
from gpaw.mpi import rank, size, world, serial_comm
from gpaw.blacs import BlacsGrid, BlacsDescriptor, Redistributor
from gpaw.response.parallel import parallel_partition, set_communicator
from gpaw.fd_operators import Gradient, Laplace
from gpaw.sphere.lebedev import weight_n, R_nv
from gpaw.io.tar import Writer, Reader
from gpaw import debug
from scipy.special import sici
from scipy.special.orthogonal import p_roots

class FXCCorrelation:

    def __init__(self,
                 calc,
                 txt=None,
                 tag='',
                 qsym=True,
                 xc=None,
                 num=0,
                 write=False,
                 method='standard',
                 lambda_points=8,
                 density_cut=None,
                 paw_correction=1,
                 unit_cells=None):
        
        self.calc = calc

        if txt is None:
            if rank == 0:
                self.txt = sys.stdout
            else:
                self.txt = devnull
        else:
            assert isinstance(txt, str)
            from ase.parallel import paropen
            self.txt = paropen(txt, 'w')
        
        self.tag = tag
        self.qsym = qsym
        self.num = num
        self.nspins = calc.wfs.nspins
        self.bz_k_points = calc.wfs.kd.bzk_kc
        self.atoms = calc.get_atoms()
        self.setups = calc.wfs.setups
        self.bz_q_points = calc.wfs.kd.get_bz_q_points(first=True)
        if qsym == False:
            self.ibz_q_points = self.bz_q_points
            self.q_weights = (np.ones(len(self.bz_q_points))
                              / len(self.bz_q_points))
        else:
            op_scc = calc.wfs.kd.symmetry.op_scc
            self.ibz_q_points = calc.wfs.kd.get_ibz_q_points(self.bz_q_points,
                                                             op_scc)[0]
            self.q_weights = calc.wfs.kd.q_weights
        
        if xc is None:
            self.xc = 'RPA'
        else:
            self.xc = xc

        self.lambda_points = lambda_points
        self.method = method
        self.density_cut = density_cut
        if self.density_cut is None:
            self.density_cut = 1.e-6
        assert paw_correction in range(4)
        self.paw_correction = paw_correction
        self.unit_cells = unit_cells
        self.print_initialization()
        self.initialized = 0

   
    def get_fxc_correlation_energy(self,
                                   kcommsize=None,
                                   directions=None,
                                   skip_gamma=False,
                                   ecut=10,
                                   nbands=None,
                                   gauss_legendre=None,
                                   frequency_cut=None,
                                   frequency_scale=None,
                                   w=None,
                                   restart=None):
            
        self.initialize_calculation(w,
                                    ecut,
                                    nbands,
                                    kcommsize,
                                    gauss_legendre,
                                    frequency_cut,
                                    frequency_scale)
        
        if directions is None:
            directions = [[0, 1/3.], [1, 1/3.], [2, 1/3.]]
        if skip_gamma:
            directions = []
                
        E_q = []
        if restart is not None:
            assert isinstance(restart, str)
            try:
                f = paropen(restart, 'r')
                lines = f.readlines()
                for line in lines:
                    E_q.append(eval(line))
                f.close()
                print('Correlation energy obtained ' \
                      +'from %s q-points obtained from restart file: ' \
                      % len(E_q), restart, file=self.txt)
                print(file=self.txt)
            except:
                IOError

        if len(E_q) == 0:
            kernel = Kernel(self.calc,
                            self.xc,
                            self.method,
                            self.ibz_q_points,
                            self.w,
                            self.ecut,
                            self.txt,
                            self.tag,
                            paw_correction=self.paw_correction,
                            unit_cells=self.unit_cells,
                            density_cut=self.density_cut)
            kernel.calculate_fhxc(directions=[d[0] for d in directions])
            del kernel
 
        for index, q in zip(range(len(E_q), len(self.ibz_q_points)),
                            self.ibz_q_points[len(E_q):]):
            if abs(np.dot(q, q))**0.5 < 1.e-5:
                E_q0 = 0.
                if skip_gamma:
                    print('Not calculating q at the Gamma point', file=self.txt)
                    print(file=self.txt)
                else:
                    for d in directions:
                        E_q0 += self.E_q(q,
                                         index=index,
                                         direction=d[0]) * d[1]
                E_q.append(E_q0)
            else:
                E_q.append(self.E_q(q, index=index))
                    
            if restart is not None:
                f = paropen(restart, 'a')
                print(E_q[-1], file=f)
                f.close()
    
        E = np.dot(np.array(self.q_weights), np.array(E_q).real)

        print('%s correlation energy:' % self.xc, file=self.txt)
        print('E_c = %s eV' % E, file=self.txt)
        print(file=self.txt)
        print('Calculation completed at:  ', ctime(), file=self.txt)
        print(file=self.txt)
        print('------------------------------------------------------', file=self.txt)
        print(file=self.txt)

        return E


    def get_E_q(self,
                kcommsize=None,
                index=None,
                q=[0., 0., 0.],
                direction=0,
                integrated=True,
                ecut=10,
                nbands=None,
                gauss_legendre=None,
                frequency_cut=None,
                frequency_scale=None,
                w=None):

        self.initialize_calculation(w,
                                    ecut,
                                    nbands,
                                    kcommsize,
                                    gauss_legendre,
                                    frequency_cut,
                                    frequency_scale)
        E_q = self.E_q(q,
                       direction=direction,
                       integrated=integrated)
        
        print('Calculation completed at:  ', ctime(), file=self.txt)
        print(file=self.txt)
        print('------------------------------------------------------', file=self.txt)
        return E_q


    def E_q(self,
            q,
            index=None,
            direction=0,
            integrated=True):

        if abs(np.dot(q, q))**0.5 < 1.e-5:
            q = [0.,0.,0.]
            q[direction] = 1.e-5
            optical_limit = True
        else:
            optical_limit = False

        ns = self.nspins

        if self.xc[0] == 'r':
            name = self.xc + '_' + self.method + '_' + self.tag
            if optical_limit:
                r = Reader('fhxc_%s_%s_0%s.gpw' % (name, self.ecut, direction))
            else:
                r = Reader('fhxc_%s_%s_%s.gpw' % (name, self.ecut, index))
        else:
            name = self.xc + '_' + self.tag
            if optical_limit:
                r = Reader('fhxc_%s_%s_0%s.gpw' % (name, self.ecut, direction))
            else:
                r = Reader('fhxc_%s_%s_%s.gpw' % (name, self.ecut, index))
                
        npw = r.dimension('sG') / ns

        if index is not None:
            print('#', index, file=self.txt)
        if optical_limit:
            print('q = [0 0 0] -', 'Polarization: ', direction, file=self.txt)
        else:
            print('q = [%1.6f %1.6f %1.6f] -' \
                % (q[0],q[1],q[2]), '%s planewaves' % npw, file=self.txt)
        print('Calculating KS response function', file=self.txt)

        if self.nbands is None:
            nbands = npw
        else:
            nbands = self.nbands

        if self.txt is sys.stdout:
            txt = 'response.txt'
        else:
            txt='response_' + self.txt.name
        df = DF(calc=self.calc,
                xc=None,
                nbands=nbands,
                eta=0.0,
                q=q,
                txt=txt,
                w=self.w * 1j,
                ecut=self.ecut,
                G_plus_q=True,
                density_cut=self.density_cut,
                kcommsize=self.kcommsize,
                comm=world,
                optical_limit=optical_limit,
                hilbert_trans=False)
        
        df.initialize()
        Nw_local = df.Nw_local

        Kc_G = np.zeros(npw, dtype=complex)

        for iG in range(npw):
            qG = np.dot(q + df.Gvec_Gc[iG], df.bcell_cv)
            Kc_G[iG] = 4 * np.pi / np.dot(qG, qG)

        chi0 = np.zeros((Nw_local, ns*npw, ns*npw), dtype=complex)

        df.calculate(seperate_spin=0)
        chi0[:, :npw, :npw] = df.chi0_wGG[:]
        if ns == 2:
            df.ecut *= Hartree
            df.xc = 'RPA'
            df.initialize()
            df.calculate(seperate_spin=1)
            chi0[:, npw:, npw:] = df.chi0_wGG[:]
        del df.chi0_wGG
                
        local_E_q_w = np.zeros(Nw_local, dtype=complex)
        E_q_w = np.empty(len(self.w), complex)

        print('Calculating interacting response function', file=self.txt)
        ls, l_ws = p_roots(self.lambda_points)
        ls = (ls + 1.0) * 0.5
        l_ws *= 0.5
        for i in range(Nw_local):
            chi0_fhxc = np.dot(chi0[i], r.get('fhxc_sGsG'))
            for l, l_w in zip(ls, l_ws):
                chi_l = np.linalg.solve(np.eye(npw*ns, npw*ns)
                                        - l*chi0_fhxc, chi0[i]).real
                for s1 in range(ns):
                    for s2 in range(ns):
                        X_ss = chi_l[s1*npw:(s1+1)*npw, s2*npw:(s2+1)*npw]
                        local_E_q_w[i] -= np.dot(np.diag(X_ss), Kc_G)*l_w
            local_E_q_w[i] += np.dot(np.diag(chi0[i]), np.tile(Kc_G, ns))
        df.wcomm.all_gather(local_E_q_w, E_q_w)

        del df, chi0, chi0_fhxc, chi_l, X_ss, Kc_G
        
        if self.gauss_legendre is not None:
            E_q = np.sum(E_q_w * self.gauss_weights * self.transform) \
                  / (4 * np.pi)
        else:
            dws = self.w[1:] - self.w[:-1]
            E_q = np.dot((E_q_w[:-1] + E_q_w[1:])/2., dws) / (2 * np.pi)

        print('E_c(q) = %s eV' % E_q.real, file=self.txt)
        print(file=self.txt)

        if integrated:
            return E_q.real
        else:
            return E_q_w.real


    def initialize_calculation(self,
                               w,
                               ecut,
                               nbands,
                               kcommsize,
                               gauss_legendre,
                               frequency_cut,
                               frequency_scale):
        if kcommsize is None:
            if len(self.calc.wfs.kd.bzk_kc) == 1:
                kcommsize = 1
            else:
                kcommsize = world.size
            
        if w is not None:
            assert (gauss_legendre is None and
                    frequency_cut is None and
                    frequency_scale is None)
        else:
            if gauss_legendre is None:
                gauss_legendre = 16
            self.gauss_points, self.gauss_weights = p_roots(gauss_legendre)
            if frequency_scale is None:
                frequency_scale = 2.0
            if frequency_cut is None:
                frequency_cut = 800.
            ys = 0.5 - 0.5 * self.gauss_points
            ys = ys[::-1]
            w = (-np.log(1-ys))**frequency_scale
            w *= frequency_cut/w[-1]
            alpha = (-np.log(1-ys[-1]))**frequency_scale/frequency_cut
            transform = (-np.log(1-ys))**(frequency_scale-1) \
                        / (1-ys)*frequency_scale/alpha
            self.transform = transform
            
        dummy = DF(calc=self.calc,
                   xc='RPA',
                   eta=0.0,
                   w=w * 1j,
                   q=[0.,0.,0.0001],
                   ecut=ecut,
                   optical_limit=True,
                   hilbert_trans=False,
                   kcommsize=kcommsize)
        dummy.txt = devnull
        dummy.initialize(simple_version=True)
            
        self.ecut = ecut
        self.w = w
        self.gauss_legendre = gauss_legendre
        self.frequency_cut = frequency_cut
        self.frequency_scale = frequency_scale
        self.kcommsize = kcommsize
        self.nbands = nbands

        if debug:
            print(file=self.txt)
            print('DEBUG MODE', file=self.txt)
            print(file=self.txt)
    
        print('Numerical coupling constant integration' \
              + ' with % s Gauss-Legendre points' % self.lambda_points, file=self.txt)
        print(file=self.txt)
        print('Planewave cutoff              : %s eV' % ecut, file=self.txt)
        print('Number of Planewaves at Gamma : %s' % dummy.npw, file=self.txt)
        if self.nbands is None:
            print('Response function bands       :' \
                  + ' Equal to number of Planewaves', file=self.txt)
        elif isinstance(self.nbands, float):
            print('Response function bands       : %s' \
                  % int(dummy.npw * self.nbands), file=self.txt)
        else:
            print('Response function bands       : %s' \
                  % self.nbands, file=self.txt)
        if self.density_cut is not None:
            print(file=self.txt)
            print('Min value of pseudo density   : %1.2e Bohr^-3'\
                  % np.min(self.calc.density.nt_sG), file=self.txt)
            print('Max value of pseudo density   : %1.2e Bohr^-3'\
                  % np.max(self.calc.density.nt_sG), file=self.txt)
            print('Density cutoff in fxc at      : %1.2e Bohr^-3'\
                  % self.density_cut, file=self.txt)
        print('Frequencies', file=self.txt)
        if self.gauss_legendre is not None:
            print('    Gauss-Legendre integration '\
                  + 'with %s frequency points' % len(self.w), file=self.txt)
            print('    Frequency cutoff is '\
                  + '%s eV and scale (B) is %s' % (self.w[-1],
                                                  self.frequency_scale), file=self.txt)
        else:
            print('    %s specified frequency points' \
                  % len(self.w), file=self.txt)
            print('    Frequency cutoff is %s eV' \
                  % self.w[-1], file=self.txt)
        print(file=self.txt)
        print('Parallelization scheme', file=self.txt)
        print('     Total CPUs        : %d' % dummy.comm.size, file=self.txt)
        if dummy.kd.nbzkpts == 1:
            print('     Band parsize      : %d' % dummy.kcomm.size, file=self.txt)
        else:
            print('     Kpoint parsize    : %d' % dummy.kcomm.size, file=self.txt)
        print('     Frequency parsize : %d' % dummy.wScomm.size, file=self.txt)
        print('Memory usage estimate', file=self.txt)
        print('     chi0_wGG(q)         : %6.3f MB / frequency point' \
              % (self.nspins**2 * dummy.npw**2 * 16. / 1024**2), file=self.txt)
        print('     Kernel_GG(q)        : %6.3f MB / CPU' \
              % ((2*self.nspins + 3%self.nspins) * dummy.npw**2 * 16. / 1024**2), file=self.txt)
        if self.method == 'standard':
            n0 = dummy.gd.N_c[0] * dummy.gd.N_c[1] * dummy.gd.N_c[2]
            print('     Kernel_rG(q) (Int.) : %6.3f MB / CPU' \
                      % ((2*self.nspins + 3%self.nspins)
                         * dummy.npw * float(n0)/world.size * 16. / 1024**2), file=self.txt)
            
        print(file=self.txt)
        del dummy


    def print_initialization(self):
        
        print('------------------------------------------------------', file=self.txt)
        print('Non-self-consistent %s correlation energy' \
              % self.xc, file=self.txt)
        if self.xc is not 'RPA':
            if self.xc[0] == 'r':
                if self.method == 'solid':
                    print('Periodic average density', file=self.txt)
                else:
                    print('Non-periodic two-point density', file=self.txt)
            if self.paw_correction == 0:
                print('Using pseudo-density with ALDA PAW corrections', file=self.txt)
            elif self.paw_correction == 1:
                print('Using all-electron density', file=self.txt)
            elif self.paw_correction == 2:
                print('Using pseudo-density with average ALDA PAW corrections', file=self.txt)
            elif self.paw_correction == 3:
                print('Using pseudo-density', file=self.txt)
        print('------------------------------------------------------', file=self.txt)
        print('Started at:  ', ctime(), file=self.txt)
        print(file=self.txt)
        print('Atoms                          :   %s' \
              % self.atoms.get_chemical_formula(mode="hill"), file=self.txt)
        print('Ground state XC functional     :   %s' \
              % self.calc.hamiltonian.xc.name, file=self.txt)
        print('Valence electrons              :   %s' \
              % self.setups.nvalence, file=self.txt)
        print('Number of Bands                :   %s' \
              % self.calc.wfs.bd.nbands, file=self.txt)
        print('Number of Converged Bands      :   %s' \
              % self.calc.input_parameters['convergence']['bands'], file=self.txt)
        print('Number of Spins                :   %s' \
              % self.nspins, file=self.txt)
        print('Number of k-points             :   %s' \
              % len(self.calc.wfs.kd.bzk_kc), file=self.txt)
        print('Number of q-points             :   %s' \
              % len(self.bz_q_points), file=self.txt)
        print('Number of Irreducible k-points :   %s' \
              % len(self.calc.wfs.kd.ibzk_kc), file=self.txt)
        if self.qsym:
            print('Number of Irreducible q-points :   %s' \
                  % len(self.ibz_q_points), file=self.txt)
        else:
            print('No reduction of q-points', file=self.txt)
        print(file=self.txt)
        for q, weight in zip(self.ibz_q_points, self.q_weights):
            print('q: [%1.4f %1.4f %1.4f] - weight: %1.3f' \
                  % (q[0],q[1],q[2], weight), file=self.txt)
        print(file=self.txt)
        print('------------------------------------------------------', file=self.txt)
        print('------------------------------------------------------', file=self.txt)
        print(file=self.txt)


    def get_C6_coefficient(self,
                           ecut=100.,
                           nbands=None,
                           kcommsize=None,
                           gauss_legendre=None,
                           frequency_cut=None,
                           frequency_scale=None,
                           direction=2):

        self.initialize_calculation(None,
                                    ecut,
                                    nbands,
                                    kcommsize,
                                    gauss_legendre,
                                    frequency_cut,
                                    frequency_scale)

        d = direction
        d_pro = []
        for i in range(3):
            if i != d:
                d_pro.append(i)
        
        q = [0.,0.,0.]
        q[d] = 1.e-5

        kernel = Kernel(self.calc,
                        self.xc,
                        self.method,
                        self.ibz_q_points,
                        self.w,
                        self.ecut,
                        self.txt,
                        self.tag,
                        paw_correction=self.paw_correction,
                        unit_cells=self.unit_cells,
                        density_cut=self.density_cut)

        kernel.calculate_fhxc(directions=[d])

        del kernel

        if self.xc[0] == 'r':
            name = self.xc + '_' + self.method + '_' + self.tag
            r = Reader('fhxc_%s_%s_0%s.gpw' % (name, self.ecut, direction))
        else:
            name = self.xc + '_' + self.tag
            r = Reader('fhxc_%s_%s_0%s.gpw' % (name, self.ecut, direction))
                
        ns = self.nspins
        npw = r.dimension('sG') / ns

        if self.nbands is None:
            nbands = npw
        else:
            nbands = self.nbands

        if self.txt is sys.stdout:
            txt = 'response.txt'
        else:
            txt='response_' + self.txt.name
        df = DF(calc=self.calc,
                xc=None,
                nbands=nbands,
                eta=0.0,
                q=q,
                txt=txt,
                w=self.w * 1j,
                ecut=self.ecut,
                comm=world,
                optical_limit=True,
                G_plus_q=True,
                kcommsize=self.kcommsize,
                hilbert_trans=False)
        
        print('Calculating %s response function' % self.xc, file=self.txt)
        print('Polarization: %s' % d, file=self.txt)

        df.initialize()
        Nw_local = df.Nw_local
        chi0 = np.zeros((Nw_local, ns*npw, ns*npw), dtype=complex)

        df.calculate(seperate_spin=0)
        chi0[:, :npw, :npw] = df.chi0_wGG[:]
        if ns == 2:
            df.ecut *= Hartree
            df.xc = 'RPA'
            df.initialize()
            df.calculate(seperate_spin=1)
            chi0[:, npw:2*npw, npw:2*npw] = df.chi0_wGG[:]
        del df.chi0_wGG
        
        local_a0_w = np.zeros(Nw_local, dtype=complex)
        a0_w = np.empty(len(self.w), complex)
        local_a_w = np.zeros(Nw_local, dtype=complex)
        a_w = np.empty(len(self.w), complex)

        Gvec_Gv = np.dot(df.Gvec_Gc + np.array(q), df.bcell_cv)
        gd = self.calc.density.gd
        n_d = gd.get_size_of_global_array()[d]
        d_d = gd.get_grid_spacings()[d]
        r_d = np.array([i*d_d for i in range(n_d)])

        print('Calculating real space integrals', file=self.txt)

        int_G = np.zeros(npw, complex)
        for iG in range(npw):
            if df.Gvec_Gc[iG, d_pro[0]] == 0 and df.Gvec_Gc[iG, d_pro[1]] == 0:
                int_G[iG] = np.sum(r_d * np.exp(1j*Gvec_Gv[iG, d] * r_d))*d_d
        int2_GG = np.outer(int_G, int_G.conj())

        print('Calculating dynamic polarizability', file=self.txt)

        for i in range(Nw_local):
            chi0_fhxc = np.dot(chi0[i], r.get('fhxc_sGsG'))
            chi = np.linalg.solve(np.eye(npw*ns, npw*ns)
                                  - chi0_fhxc, chi0[i]).real
            for s1 in range(ns):
                X0 = chi0[i, s1*npw:(s1+1)*npw, s1*npw:(s1+1)*npw]
                local_a0_w[i] += np.trace(np.dot(X0, int2_GG))
                for s2 in range(ns):
                    X_ss = chi[s1*npw:(s1+1)*npw, s2*npw:(s2+1)*npw]
                    local_a_w[i] += np.trace(np.dot(X_ss, int2_GG))
        df.wcomm.all_gather(local_a0_w, a0_w)
        df.wcomm.all_gather(local_a_w, a_w)

        A = df.vol / gd.cell_cv[d,d]
        a0_w *= A**2 / df.vol
        a_w *= A**2 / df.vol

        C06 = np.sum(a0_w**2 * self.gauss_weights
                     * self.transform) * 3 / (2*np.pi)
        C6 = np.sum(a_w**2 * self.gauss_weights
                    * self.transform) * 3 / (2*np.pi)

        print('C06 = %s Ha*Bohr**6' % (C06.real / Hartree), file=self.txt)
        print('C6 = %s Ha*Bohr**6' % (C6.real / Hartree), file=self.txt)
        print(file=self.txt)

        return C6.real / Hartree, C06.real / Hartree


class Kernel:
    
    def __init__(self, calc, xc, method, q_points, w, ecut, txt, tag,
                 paw_correction=1, unit_cells=None, density_cut=None):

        self.calc = calc
        self.xc = xc
        self.method = method
        self.q_points = q_points
        self.ecut = ecut
        self.txt = txt
        self.tag = tag
        self.paw_correction = paw_correction
        self.unit_cells = unit_cells
        self.density_cut = density_cut
        self.Gvec_qGc = {}
        self.npw_q = []

        self.A_x = -(3/4.) * (3/np.pi)**(1/3.)

        if calc.wfs.nspins == 1:
            self.n_g = calc.get_all_electron_density(gridrefinement=1)
        else:
            self.n_g = (calc.get_all_electron_density(gridrefinement=1, spin=0)
                     + calc.get_all_electron_density(gridrefinement=1, spin=1))
        self.n_g *= Bohr**3
        
        if xc == 'rAPBE':
            if calc.wfs.nspins == 1:
                nf_g = calc.get_all_electron_density(gridrefinement=2)
            else:
                nf_g = (calc.get_all_electron_density(gridrefinement=2, spin=0)
                     + calc.get_all_electron_density(gridrefinement=2, spin=1))
            nf_g *= Bohr**3
            gdf = calc.density.gd.refine()
            grad_v = [Gradient(gdf, v, n=1).apply for v in range(3)]
            #laplace = Laplace(gdf, n=1).apply
            gradnf_vg = gdf.empty(3)
            for v in range(3):
                grad_v[v](nf_g, gradnf_vg[v])
            self.gradn_vg = gradnf_vg[:,::2,::2,::2]
            
        for iq, q in enumerate(q_points):
            dummy = DF(calc=self.calc,
                       q=q,
                       hilbert_trans=False,
                       w=1j*w,
                       ecut=ecut,
                       G_plus_q=True)
            dummy.txt = devnull
            dummy.initialize(simple_version=True)
            self.Gvec_qGc[iq] = dummy.Gvec_Gc
            self.npw_q.append(len(dummy.Gvec_Gc))
        self.gd = dummy.gd
        self.nG = dummy.gd.N_c
        self.vol = dummy.vol
        self.bcell_cv = dummy.bcell_cv
        self.acell_cv = dummy.acell_cv
        del dummy

    def calculate_fhxc(self, directions=[0,1,2]):

        print('Calculating %s kernel' % self.xc, file=self.txt)

        if self.xc == 'RPA':
            self.calculate_rpa_kernel(directions)
            
        else:
            if self.xc[0] == 'r':
                if self.method == 'solid':
                    self.calculate_rkernel_solid(directions)
                elif self.method == 'standard':
                    self.calculate_rkernel(directions)
                else:
                    raise 'Method % s not known' % self.method
            else:
                self.calculate_local_kernel()


    def calculate_rpa_kernel(self, directions=[0,1,2]):

        Gvec_qGc = self.Gvec_qGc
        npw_q = self.npw_q
        bcell_cv = self.bcell_cv

        ns = self.calc.wfs.nspins
        
        for iq, q in enumerate(self.q_points):
            if abs(np.dot(q, q))**0.5 < 1.e-5:
                iq0 = iq
                continue
            Gvec_Gc = Gvec_qGc[iq]
            npw = npw_q[iq]
            qG_Gv = np.dot(q + Gvec_Gc, bcell_cv)

            Kc_GG = np.zeros((npw, npw), dtype=float)
            for iG in range(npw):
                Kc_GG[iG,iG] = 4 * np.pi / np.dot(qG_Gv[iG], qG_Gv[iG])

            fhxc_sGsG = np.tile(Kc_GG, (ns, ns))
            
            if rank == 0:
                w = Writer('fhxc_RPA_%s_%s_%s.gpw' % (self.tag, self.ecut, iq))
                w.dimension('sG', ns*npw)
                w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                w.fill(fhxc_sGsG)
                w.close()
            world.barrier()

        # Gamma term
        Gvec_Gc = Gvec_qGc[iq0]
        npw = npw_q[iq0]
        for d in directions:
            q = np.array([0., 0., 0.])
            q[d] = 1.e-5
            qG_Gv = np.dot(q + Gvec_Gc, bcell_cv)
            Kc_GG = np.zeros((npw, npw), dtype=float)
            for iG in range(npw):
                Kc_GG[iG,iG] = 4 * np.pi / np.dot(qG_Gv[iG], qG_Gv[iG])
            
            fhxc_sGsG = np.tile(Kc_GG, (ns, ns))

            if rank == 0:
                w = Writer('fhxc_RPA_%s_%s_0%s.gpw' % (self.tag, self.ecut, d))
                w.dimension('sG', ns*npw)
                w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                w.fill(fhxc_sGsG)
                w.close()
            world.barrier()
        
        print(file=self.txt)


    def calculate_rkernel(self, directions=[0,1,2]):

        Gvec_qGc = self.Gvec_qGc
        npw_q = self.npw_q
        gd = self.gd
        nG = self.nG
        vol = self.vol
        bcell_cv = self.bcell_cv
        acell_cv = self.acell_cv
        
        ns = self.calc.wfs.nspins
        n_g = self.n_g

        fx_g = ns * self.get_fxc_g(n_g)
        qc_g = (-4 * np.pi * ns / fx_g)**0.5
        flocal_g = qc_g**3 * fx_g / (6 * np.pi**2)
        #flocal_g = 4 * n_g * fx_g # LDA
        Vlocal_g = 2 * qc_g / np.pi
        #Vlocal_g = 4 * (3 * n_g / np.pi)**(1./3.)
        
        nG0 = np.prod(nG)
        r_vg = gd.get_grid_point_coordinates()
        r_vgx = r_vg[0].flatten()
        r_vgy = r_vg[1].flatten()
        r_vgz = r_vg[2].flatten()

        # Unit cells
        R = []
        R_weight = []
        if self.unit_cells is None:
            N_R = self.calc.wfs.kd.N_c
        else:
            N_R = self.unit_cells
        N_R0 = N_R[0]*N_R[1]*N_R[2]
        for i in range(-N_R[0]+1, N_R[0]):
            for j in range(-N_R[1]+1, N_R[1]):
                for h in range(-N_R[2]+1, N_R[2]):
                    R.append(i*acell_cv[0] + j*acell_cv[1] + h*acell_cv[2])
                    R_weight.append((N_R[0]-abs(i))*
                                    (N_R[1]-abs(j))*
                                    (N_R[2]-abs(h)) / float(N_R0))
        if N_R0 > 1:
            dv = self.calc.density.gd.dv
            gc = (3.*dv/4./np.pi)**(1./3.)
            Vlocal_g -= 2*np.pi * gc**2 /dv
            print('    Lattice point sampling: ' \
                  + '(%s x %s x %s)^2 ' % (N_R[0], N_R[1], N_R[2]) \
                  + ' Reduced to %s lattice points' % len(R), file=self.txt)
        
        l_g_size = -(-nG0 // world.size)
        l_g_range = range(world.rank * l_g_size,
                          min((world.rank+1) * l_g_size, nG0))

        fhxc_qsGr = {}
        for iq in range(len(npw_q)):
            fhxc_qsGr[iq] = np.zeros((ns, npw_q[iq], len(l_g_range)),
                                     dtype=complex)

        inv_error = np.seterr()
        np.seterr(invalid='ignore')
        np.seterr(divide='ignore')

        t0 = time()
        # Loop over Lattice points
        for i, R_i in enumerate(R):
            # Loop over r'.
            # f_rr, V_rr and V_off are functions of r (dim. as r_vg[0])
            if i == 1:
                print('      Finished 1 cell in %s seconds - estimated %s seconds left' % (int(time() - t0), int(len(R)*(time()-t0))), file=self.txt)
            if len(R) > 5:
                if (i+1) % (len(R)/5+1) == 0:
                    print('      Finished %s cells in %s seconds - estimated %s seconds left' % (i, int(time() - t0), int((len(R)-i)*(time() - t0)/i)), file=self.txt)
            for g in l_g_range:
                r_x = r_vgx[g] + R_i[0]
                r_y = r_vgy[g] + R_i[1]
                r_z = r_vgz[g] + R_i[2]

                # |r-r'-R_i|
                rr = ((r_vg[0]-r_x)**2 +
                      (r_vg[1]-r_y)**2 +
                      (r_vg[2]-r_z)**2)**0.5

                n_av = (n_g + n_g.flatten()[g]) / 2.
                fx_g = ns * self.get_fxc_g(n_av, index=g)
                qc_g = (-4 * np.pi * ns / fx_g)**0.5
                x = qc_g * rr
                #kf_g = (3 * np.pi**2 * n_av)**(1./3.)
                #x = 2 * kf_g * rr  # LDA
                f_rr = fx_g * (np.sin(x) - x*np.cos(x)) \
                              / (2 * np.pi**2 * rr**3)
                if N_R0 > 1:
                    V_rr = (sici(x)[0] * 2 / np.pi - 1) / rr
                else:
                    V_rr = (sici(x)[0] * 2 / np.pi) / rr
                    
                # Terms with r = r'
                if (np.abs(R_i) < 0.001).all():
                    tmp_flat = f_rr.flatten()
                    tmp_flat[g] = flocal_g.flatten()[g]
                    f_rr = tmp_flat.reshape(nG)
                    tmp_flat = V_rr.flatten()
                    tmp_flat[g] = Vlocal_g.flatten()[g]
                    V_rr = tmp_flat.reshape(nG)
                    del tmp_flat

                f_rr[np.where(n_av < self.density_cut)] = 0.0
                V_rr[np.where(n_av < self.density_cut)] = 0.0

                f_rr *= R_weight[i]
                V_rr *= R_weight[i]

                # r-r'-R_i
                r_r = np.array([r_vg[0]-r_x, r_vg[1]-r_y, r_vg[2]-r_z])

                # Fourier transform of r
                for iq, q in enumerate(self.q_points):
                    q_v = np.dot(q, bcell_cv)
                    e_q = np.exp(-1j * gemmdot(q_v, r_r, beta=0.0))
                    tmp_fhxc = np.fft.fftn((f_rr+V_rr)*e_q) * vol / nG0
                    if ns == 2:
                        tmp_V_off = np.fft.fftn(V_rr*e_q) * vol / nG0
                    for iG in range(npw_q[iq]):
                        assert (nG / 2 - np.abs(Gvec_qGc[iq][iG]) > 0).all()
                        f_i = Gvec_qGc[iq][iG] % nG
                        fhxc_qsGr[iq][0, iG, g-l_g_range[0]] += \
                                      tmp_fhxc[f_i[0], f_i[1], f_i[2]]
                        if ns == 2:
                            fhxc_qsGr[iq][1, iG, g-l_g_range[0]] += \
                                          tmp_V_off[f_i[0], f_i[1], f_i[2]]
        world.barrier()
        
        np.seterr(**inv_error)

        for iq, q in enumerate(self.q_points):
            npw = npw_q[iq]
            Gvec_Gc = Gvec_qGc[iq]
            fhxc_sGsG = np.zeros((ns*npw, ns*npw), complex)
            l_pw_size = -(-npw // world.size)
            l_pw_range = range(world.rank * l_pw_size,
                               min((world.rank+1) * l_pw_size, npw))

            if world.size > 1 :
                bg1 = BlacsGrid(world, 1, world.size)
                bg2 = BlacsGrid(world, world.size, 1)
                bd1 = bg1.new_descriptor(npw, nG0, npw, - (-nG0 / world.size))
                bd2 = bg2.new_descriptor(npw, nG0, -(-npw / world.size), nG0)

                fhxc_Glr = np.zeros((len(l_pw_range), nG0), dtype=complex)
                if ns == 2:
                    Koff_Glr = np.zeros((len(l_pw_range), nG0), dtype=complex)

                r = Redistributor(bg1.comm, bd1, bd2)
                r.redistribute(fhxc_qsGr[iq][0], fhxc_Glr, npw, nG0)
                if ns == 2:
                    r.redistribute(fhxc_qsGr[iq][1], Koff_Glr, npw, nG0)
            else:
                fhxc_Glr = fhxc_qsGr[iq][0]
                if ns == 2:
                    Koff_Glr = fhxc_qsGr[iq][1]

            # Fourier transform of r'
            for iG in range(len(l_pw_range)):
                tmp_fhxc = np.fft.fftn(fhxc_Glr[iG].reshape(nG)) * vol/nG0
                if ns == 2:
                    tmp_Koff = np.fft.fftn(Koff_Glr[iG].reshape(nG)) * vol/nG0
                for jG in range(npw):
                    assert (nG / 2 - np.abs(Gvec_Gc[jG]) > 0).all()
                    f_i = -Gvec_Gc[jG] % nG
                    fhxc_sGsG[l_pw_range[0] + iG, jG] = \
                                    tmp_fhxc[f_i[0], f_i[1], f_i[2]]
                    if ns == 2:
                        fhxc_sGsG[npw + l_pw_range[0] + iG, jG] += \
                                      tmp_Koff[f_i[0], f_i[1], f_i[2]]

            if ns == 2:
                fhxc_sGsG[:npw, npw:] = fhxc_sGsG[npw:, :npw]
                fhxc_sGsG[npw:, npw:] = fhxc_sGsG[:npw, :npw]

            world.sum(fhxc_sGsG)

            if rank == 0:
                if abs(np.dot(q, q))**0.5 < 1.e-5:
                    for d in directions:
                        q0 = np.array([0., 0., 0.])
                        q0[d] = 1.e-5

                        w = Writer('fhxc_%s_%s_%s_%s_0%s.gpw' % (self.xc,
                                                                 self.method,
                                                                 self.tag,
                                                                 self.ecut,
                                                                 d))
                        w.dimension('sG', ns*npw)
                        w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                        if N_R0 > 1:
                            Gq_Gv = np.dot(Gvec_Gc + q0, bcell_cv)
                            Gq_G = (Gq_Gv[:,0]**2 + Gq_Gv[:,1]**2 + Gq_Gv[:,2]**2)**0.5
                            vq_G = 4 * np.pi / Gq_G**2
                            w.fill(fhxc_sGsG / vol
                                   + np.tile(np.eye(npw) * vq_G, (ns, ns)))
                        else:
                            w.fill(fhxc_sGsG / vol)
                        w.close()
                else:
                    w = Writer('fhxc_%s_%s_%s_%s_%s.gpw' % (self.xc,
                                                            self.method,
                                                            self.tag,
                                                            self.ecut,
                                                            iq))
                    w.dimension('sG', ns*npw)
                    w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                    if N_R0 > 1:
                        Gq_Gv = np.dot(Gvec_Gc + q, bcell_cv)
                        Gq_G = (Gq_Gv[:,0]**2 + Gq_Gv[:,1]**2 + Gq_Gv[:,2]**2)**0.5
                        vq_G = 4 * np.pi / Gq_G**2
                        w.fill(fhxc_sGsG / vol
                               + np.tile(np.eye(npw) * vq_G, (ns, ns)))
                    else:
                        w.fill(fhxc_sGsG / vol)
                    w.close()
            world.barrier()

        print(file=self.txt)

        
    def calculate_rkernel_solid(self, directions=[0,1,2]):

        Gvec_qGc = self.Gvec_qGc
        npw_q = self.npw_q
        gd = self.gd
        nG = self.nG
        vol = self.vol
        bcell_cv = self.bcell_cv

        ns = self.calc.wfs.nspins

        n_g = self.n_g

        fx_g = ns * self.get_fxc_g(n_g)
        fx_g[np.where(n_g < self.density_cut)] = 0.0
        qc_g = (-4 * np.pi * ns / fx_g)**0.5
        #kf_g = (3 * np.pi**2 * n_g)**(1./3.)

        r_vg = gd.get_grid_point_coordinates()

        for iq, q in enumerate(self.q_points):
            if abs(np.dot(q, q))**0.5 < 1.e-5:
                iq0 = iq
                continue
            npw = npw_q[iq]
            Gvec_Gc = Gvec_qGc[iq]
            fhxc_sGsG = np.zeros((ns*npw, ns*npw), dtype=complex)
            l_pw_size = -(-npw // world.size)
            l_pw_range = range(world.rank * l_pw_size,
                               min((world.rank+1) * l_pw_size, npw))

            for iG in l_pw_range:
                for jG in range(npw):
                    dGq_c = (Gvec_Gc[iG] + Gvec_Gc[jG])/ 2. + q
                    if (np.abs(dGq_c) < 1.e-12).all():
                        dGq_c = np.array([0.0, 0.0, 0.0001])
                    dGq_v = np.dot(dGq_c, bcell_cv)
                    fx = fx_g.copy()
                    #fx[np.where(2*kf_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0
                    fx[np.where(qc_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0
                    v_c = 4 * np.pi * np.ones(np.shape(fx_g), float)
                    v_c /= np.dot(dGq_v, dGq_v)
                    #v_c[np.where(2*kf_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0
                    v_c[np.where(qc_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0

                    dG_c = Gvec_Gc[iG] - Gvec_Gc[jG]
                    dG_v = np.dot(dG_c, bcell_cv)
                    dGr_g = gemmdot(dG_v, r_vg, beta=0.0)

                    fhxc_sGsG[iG, jG] = gd.integrate(np.exp(-1j*dGr_g) \
                                                     * (fx + v_c))
                    if ns == 2:
                        fhxc_sGsG[npw + iG, jG] = \
                                      gd.integrate(np.exp(-1j*dGr_g) * v_c)
            if ns == 2:
                fhxc_sGsG[:npw, npw:] = fhxc_sGsG[npw:, :npw]
                fhxc_sGsG[npw:, npw:] = fhxc_sGsG[:npw, :npw]

            world.sum(fhxc_sGsG)

            if rank == 0:
                w = Writer('fhxc_%s_%s_%s_%s_%s.gpw' % (self.xc,
                                                        self.method,
                                                        self.tag,
                                                        self.ecut,
                                                        iq))
                w.dimension('sG', ns*npw)
                w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                w.fill(fhxc_sGsG / vol)
                w.close()
            world.barrier()

        # Gamma term
        Gvec_Gc = Gvec_qGc[iq0]
        npw = npw_q[iq0]
        l_pw_size = -(-npw // world.size)
        l_pw_range = range(world.rank * l_pw_size,
                           min((world.rank+1) * l_pw_size, npw))
        for d in directions:
            q = np.array([0., 0., 0.])
            q[d] = 1.e-5
            qG_Gv = np.dot(q + Gvec_Gc, bcell_cv)
            fhxc_sGsG = np.zeros((ns*npw, ns*npw), dtype=complex)

            for iG in l_pw_range:
                for jG in range(npw):
                    dGq_c = (Gvec_Gc[iG] + Gvec_Gc[jG])/ 2. + q
                    if (np.abs(dGq_c) < 1.e-12).all():
                        dGq_c = np.array([0.0, 0.0, 0.0001])
                    dGq_v = np.dot(dGq_c, bcell_cv)
                    fx = fx_g.copy()
                    #fx[np.where(2*kf_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0
                    fx[np.where(qc_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0
                    v_c = 4 * np.pi * np.ones(np.shape(fx_g), float)
                    v_c /= np.dot(dGq_v, dGq_v)
                    #v_c[np.where(2*kf_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0
                    v_c[np.where(qc_g < np.dot(dGq_v, dGq_v)**0.5)] = 0.0

                    dG_c = Gvec_Gc[iG] - Gvec_Gc[jG]
                    dG_v = np.dot(dG_c, bcell_cv)
                    dGr_g = gemmdot(dG_v, r_vg, beta=0.0)

                    fhxc_sGsG[iG, jG] = gd.integrate(np.exp(-1j*dGr_g) * (fx + v_c))
                    if ns == 2:
                        fhxc_sGsG[npw + iG, jG] = gd.integrate(np.exp(-1j*dGr_g) * v_c)
            if ns == 2:
                fhxc_sGsG[:npw, npw:] = fhxc_sGsG[npw:, :npw]
                fhxc_sGsG[npw:, npw:] = fhxc_sGsG[:npw, :npw]

            world.sum(fhxc_sGsG)

            if rank == 0:
                w = Writer('fhxc_%s_%s_%s_%s_0%s.gpw' % (self.xc,
                                                         self.method,
                                                         self.tag,
                                                         self.ecut,
                                                         d))
                w.dimension('sG', ns*npw)
                w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                w.fill(fhxc_sGsG / vol)
                w.close()
            world.barrier()

        print(file=self.txt)

    def calculate_local_kernel(self):
        # Standard ALDA exchange kernel
        # Use with care. Results are very difficult to converge
        # Sensitive to densitycut
        ns = self.calc.wfs.nspins
        gd = self.gd

        n_g = self.n_g

        A_x = -(3/4.) * (3/np.pi)**(1/3.)
        fxc_sg = ns * (4 / 9.) * A_x * (ns*n_g)**(-2/3.)
        fxc_sg[np.where(n_g < self.density_cut)] = 0.0

        r_vg = gd.get_grid_point_coordinates()

        for iq in range(len(self.q_points)):
            npw = self.npw_q[iq]
            Gvec_Gc = self.Gvec_qGc[iq]
            l_pw_size = -(-npw // world.size)
            l_pw_range = range(world.rank * l_pw_size,
                               min((world.rank+1) * l_pw_size, npw))
            fhxc_sGsG = np.zeros((ns*npw, ns*npw), dtype=complex)
            for s in range(ns):
                for iG in l_pw_range:
                    for jG in range(npw):
                        fxc = fxc_sg[s].copy()
                        dG_c = Gvec_Gc[iG] - Gvec_Gc[jG]
                        dG_v = np.dot(dG_c, self.bcell_cv)
                        dGr_g = gemmdot(dG_v, r_vg, beta=0.0)
                        fhxc_sGsG[s*npw+iG, s*npw+jG] = \
                                            gd.integrate(np.exp(-1j*dGr_g)*fxc)

            world.sum(fhxc_sGsG)

            Kc_GG = np.zeros((npw, npw), dtype=complex)
            for iG in range(npw):
                qG_Gv = np.dot(Gvec_Gc[iG] + self.q_points[iq], self.bcell_cv)
                Kc_GG[iG, iG] = 4 * np.pi / np.dot(qG_Gv, qG_Gv)
            fhxc_sGsG += np.tile(Kc_GG, (ns, ns)) * self.vol
            
            if self.paw_correction in [0, 2]:
                print('    Calculating PAW correction', file=self.txt)
                f_paw_sGG = self.add_paw_correction(npw,
                                                    Gvec_Gc,
                                                    bcell_cv,
                                                    self.calc.wfs.setups,
                                                    self.calc.density.D_asp,
                                                    self.atoms.positions/Bohr)
                for s in range(ns):
                    fhxc_sGsG[s*npw:(s+1)*npw, s*npw:(s+1)*npw] += f_paw_sGG[s]

            if rank == 0:
                w = Writer('fhxc_%s_%s_%s_%s.gpw' % (self.xc,
                                                     self.tag,
                                                     self.ecut,
                                                     iq))
                w.dimension('sG', ns*npw)
                w.add('fhxc_sGsG', ('sG', 'sG'), dtype=complex)
                w.fill(fhxc_sGsG / self.vol)
                w.close()
            world.barrier()

        print(file=self.txt)


    def add_paw_correction(self, npw, Gvec_Gc, bcell_cv, setups, D_asp, R_av):
        # By default only used for ALDA
        ns = self.nspins
        A_x = -(3/4.) * (3/np.pi)**(1/3.)
        KxcPAW_sGG = np.zeros((ns, npw, npw), complex)
        dG_GGv = np.zeros((npw, npw, 3))
        for iG in range(npw):
            for jG in range(npw):
                dG_c = Gvec_Gc[iG] - Gvec_Gc[jG]
                dG_GGv[iG, jG] =  np.dot(dG_c, bcell_cv)

        for a, setup in enumerate(setups):
            rgd = setup.xc_correction.rgd
            ng = len(rgd.r_g)
            myng = -(-ng // world.size)
            n_qg = setup.xc_correction.n_qg
            nt_qg = setup.xc_correction.nt_qg
            nc_g = setup.xc_correction.nc_g
            nct_g = setup.xc_correction.nct_g
            Y_nL = setup.xc_correction.Y_nL
            dv_g = rgd.dv_g

            D_sp = D_asp[a]
            B_pqL = setup.xc_correction.B_pqL
            D_sLq = np.inner(D_sp, B_pqL.T)

            f_sg = rgd.empty(ns)
            ft_sg = rgd.empty(ns)

            n_sLg = np.dot(D_sLq, n_qg)
            nt_sLg = np.dot(D_sLq, nt_qg)

            # Add core density
            n_sLg[:, 0] += (4 * np.pi)**0.5 / ns * nc_g
            nt_sLg[:, 0] += (4 * np.pi)**0.5 / ns * nct_g

            coefatoms_GG = np.exp(-1j * np.inner(dG_GGv, R_av[a]))
            w = weight_n

            if self.paw_correction == 2:
                Y_nL = [Y_nL[0]]
                w = [1.]
                
            for n, Y_L in enumerate(Y_nL):
                f_sg[:] = 0.0
                n_sg = np.dot(Y_L, n_sLg)
                f_sg = ns * (4 / 9.) * A_x * (ns*n_sg)**(-2/3.)
                if self.density_cut is not None:
                    f_sg[np.where(ns*n_sg < self.density_cut)] = 0.0

                ft_sg[:] = 0.0
                nt_sg = np.dot(Y_L, nt_sLg)
                ft_sg = ns * (4 / 9.) * A_x * (ns*nt_sg)**(-2/3.)
                if self.density_cut is not None:
                    ft_sg[np.where(ns*nt_sg < self.density_cut)] = 0.0

                for i in range(world.rank * myng,
                               min((world.rank + 1) * myng, ng)):
                    coef_GG = np.exp(-1j * np.inner(dG_GGv, R_nv[n])
                                     * rgd.r_g[i])
                    for s in range(len(f_sg)):
                        KxcPAW_sGG[s] += w[n] * np.dot(coef_GG,
                                            (f_sg[s,i]-ft_sg[s,i]) * dv_g[i]) \
                                                       * coefatoms_GG

        world.sum(KxcPAW_sGG)

        return KxcPAW_sGG
                

    def get_fxc_g(self, n_g, index=None):

        if self.xc[2:] == 'LDA':
            return self.get_lda_g(n_g)
        else:
            return self.get_pbe_g(n_g, index=index)
    

    def get_lda_g(self, n_g):

        return (4. / 9.) * self.A_x * n_g**(-2./3.)


    def get_pbe_g(self, n_g, index=None):

        if index is None:
            gradn_vg = self.gradn_vg
        else:
            gradn_vg = self.calc.density.gd.empty(3)
            for v in range(3):
                gradn_vg[v] = (self.gradn_vg[v] +
                               self.gradn_vg[v].flatten()[index]) / 2

        kf_g = (3. * np.pi**2 * n_g)**(1./3.)
        s2_g = np.zeros_like(n_g)
        for v in range(3):
            axpy(1.0, gradn_vg[v]**2, s2_g)
        s2_g /= 4 * kf_g**2 * n_g**2

        e_g = self.A_x * n_g**(4./3.)
        v_g = (4. / 3.) * e_g / n_g
        f_g = (1. / 3.) * v_g / n_g

        kappa = 0.804
        mu = 0.2195149727645171

        denom_g = (1 + mu * s2_g / kappa)
        F_g = 1. + kappa - kappa / denom_g
        Fn_g = -mu / denom_g**2 * 8 * s2_g / (3 * n_g)
        Fnn_g = -11 * Fn_g / (3 * n_g) - 2 * Fn_g**2 / kappa
            
        fxc_g = f_g * F_g
        fxc_g += 2 * v_g * Fn_g
        fxc_g += e_g * Fnn_g

        #Fgrad_vg = np.zeros_like(gradn_vg)
        #Fngrad_vg = np.zeros_like(gradn_vg)
        #for v in range(3):
        #    axpy(1.0, mu / den_g**2 * gradn_vg[v] / (2 * kf_g**2 * n_g**2),
        #         Fgrad_vg[v])
        #    axpy(-8.0, Fgrad_vg[v] / (3 * n_g), Fngrad_vg[v])
        #    axpy(-2.0, Fgrad_vg[v] * Fn_g / kappa, Fngrad_vg[v])


        #tmp = np.zeros_like(fxc_g)
        #tmp1 = np.zeros_like(fxc_g)

        #for v in range(3):
            #self.grad_v[v](Fgrad_vg[v], tmp)
            #axpy(-2.0, tmp * v_g, fxc_g)
            #for u in range(3):
                #self.grad_v[u](Fgrad_vg[u] * tmp, tmp1)
                #axpy(-4.0/kappa, tmp1 * e_g, fxc_g)
            #self.grad_v[v](Fngrad_vg[v], tmp)
            #axpy(-2.0, tmp * e_g, fxc_g)
        #self.laplace(mu / den_g**2 / (2 * kf_g**2 * n_g**2), tmp)
        #axpy(1.0, tmp * e_g, fxc_g)
        
        return fxc_g


    def get_fxc_libxc_g(self, n_g):
        
        gd = self.calc.density.gd.refine()

        xc = XC('GGA_X_' + self.xc[2:])
        #xc = XC('LDA_X')
        #sigma = np.zeros_like(n_g).flat[:]
        xc.set_grid_descriptor(gd)
        sigma_xg, gradn_svg = xc.calculate_sigma(np.array([n_g]))

        dedsigma_xg = np.zeros_like(sigma_xg)
        e_g = np.zeros_like(n_g)
        v_sg = np.array([np.zeros_like(n_g)])
        
        xc.calculate_gga(e_g, np.array([n_g]), v_sg, sigma_xg, dedsigma_xg)

        sigma = sigma_xg[0].flat[:]
        gradn_vg = gradn_svg[0]
        dedsigma_g = dedsigma_xg[0]
        
        libxc = LibXC('GGA_X_' + self.xc[2:])
        #libxc = LibXC('LDA_X')
        libxc.initialize(1)
        libxc_fxc = libxc.xc.calculate_fxc_spinpaired
        
        fxc_g = np.zeros_like(n_g).flat[:]
        d2edndsigma_g = np.zeros_like(n_g).flat[:]
        d2ed2sigma_g = np.zeros_like(n_g).flat[:]

        libxc_fxc(n_g.flat[:], fxc_g, sigma, d2edndsigma_g, d2ed2sigma_g)
        fxc_g = fxc_g.reshape(np.shape(n_g))
        d2edndsigma_g = d2edndsigma_g.reshape(np.shape(n_g))
        d2ed2sigma_g = d2ed2sigma_g.reshape(np.shape(n_g))

        tmp = np.zeros_like(fxc_g)
        tmp1 = np.zeros_like(fxc_g)
        
        #for v in range(3):
            #self.grad_v[v](d2edndsigma_g * gradn_vg[v], tmp)
            #axpy(-4.0, tmp, fxc_g)
            
        #for u in range(3):
            #for v in range(3):
                #self.grad_v[v](d2ed2sigma_g * gradn_vg[u] * gradn_vg[v], tmp)
                #self.grad_v[u](tmp, tmp1)
                #axpy(4.0, tmp1, fxc_g)

        #self.laplace(dedsigma_g, tmp)
        #axpy(2.0, tmp, fxc_g)
            
        return fxc_g[::2,::2,::2]


    def get_numerical_fxc_sg(self, n_sg):

        gd = self.calc.density.gd.refine()

        delta = 1.e-4
        if self.xc[2:] == 'LDA':
            xc = XC('LDA_X')
            v1xc_sg = np.zeros_like(n_sg)
            v2xc_sg = np.zeros_like(n_sg)
            xc.calculate(gd, (1+delta)*n_sg, v1xc_sg)
            xc.calculate(gd, (1-delta)*n_sg, v2xc_sg)
            fxc_sg = (v1xc_sg - v2xc_sg) / (2 * delta * n_sg)
        else:
            fxc_sg = np.zeros_like(n_sg)
            xc = XC('GGA_X_' + self.xc[2:])
            vxc_sg = np.zeros_like(n_sg)
            xc.calculate(gd, n_sg, vxc_sg)
            for s in range(len(n_sg)):
                for x in range(len(n_sg[0])):
                    for y in range(len(n_sg[0,0])):
                        for z in range(len(n_sg[0,0,0])):
                            v1xc_sg = np.zeros_like(n_sg)
                            n1_sg = n_sg.copy()
                            n1_sg[s,x,y,z] *= (1 + delta)
                            xc.calculate(gd, n1_sg, v1xc_sg)
                            fxc_sg[s,x,y,z] = \
                (v1xc_sg[s,x,y,z] - vxc_sg[s,x,y,z]) / (delta * n_sg[s,x,y,z])
                            
        return fxc_sg[:,::2,::2,::2]



from __future__ import print_function
import os
import platform
import tempfile
import warnings
from optparse import OptionParser

import gpaw.mpi as mpi
from gpaw.cli.info import info
from gpaw import debug, __version__


def main(args=None):
    description = ('Run the GPAW test suite.  The test suite can be run in '
                   'parallel with MPI through gpaw-python.  The test suite '
                   'supports 1, 2, 4 or 8 CPUs although some tests are '
                   'skipped for some parallelizations.  If no TESTs are '
                   'given, run all tests supporting the parallelization.')

    parser = OptionParser(usage='%prog [OPTION...] [TEST...]',
                          description=description,
                          version='%%prog %s' % __version__)
    parser.add_option('-x', '--exclude',
                      type='string', default=None,
                      help='Exclude tests (comma separated list of tests).',
                      metavar='test1.py,test2.py,...')
    parser.add_option('-f', '--run-failed-tests-only',
                      action='store_true',
                      help='Run failed tests only.')
    parser.add_option('--from', metavar='TESTFILE', dest='from_test',
                      help='Run remaining tests, starting from TESTFILE')
    parser.add_option('--after', metavar='TESTFILE', dest='after_test',
                      help='Run remaining tests, starting after TESTFILE')
    parser.add_option('--range',
                      type='string', default=None,
                      help='Run tests in range test_i.py to test_j.py '
                      '(inclusive)',
                      metavar='test_i.py,test_j.py')
    parser.add_option('-j', '--jobs', type='int', default=1,
                      help='Run JOBS threads.  Each test will be executed '
                      'in serial by one thread.  This option cannot be used '
                      'for parallelization together with MPI.')
    parser.add_option('--reverse', action='store_true',
                      help=('Run tests in reverse order (less overhead with '
                            'multiple jobs)'))
    parser.add_option('-k', '--keep-temp-dir', action='store_true',
                      dest='keep_tmpdir',
                      help='Do not delete temporary files.')
    parser.add_option('-d', '--directory', help='Run test in this directory')
    parser.add_option('-s', '--show-output', action='store_true',
                      help='Show standard output from tests.')

    opt, tests = parser.parse_args(args)

    if len(tests) == 0:
        from gpaw.test import tests

    if opt.reverse:
        tests.reverse()

    if opt.run_failed_tests_only:
        tests = [line.strip() for line in open('failed-tests.txt')]

    exclude = []
    if opt.exclude is not None:
        exclude += opt.exclude.split(',')

    if opt.from_test:
        fromindex = tests.index(opt.from_test)
        tests = tests[fromindex:]

    if opt.after_test:
        index = tests.index(opt.after_test) + 1
        tests = tests[index:]

    if opt.range:
        # default start(stop) index is first(last) test
        indices = opt.range.split(',')
        try:
            start_index = tests.index(indices[0])
        except ValueError:
            start_index = 0
        try:
            stop_index = tests.index(indices[1]) + 1
        except ValueError:
            stop_index = len(tests)
        tests = tests[start_index:stop_index]

    if opt.jobs > 1:
        exclude.append('maxrss.py')

    for test in exclude:
        if test in tests:
            tests.remove(test)

    from gpaw.test import TestRunner

    if mpi.world.size > 8:
        if mpi.rank == 0:
            message = (
                '!!!!!!!\n'
                'GPAW regression test suite was not designed to run on more\n'
                'than 8 MPI tasks. Re-run test suite using 1, 2, 4 or 8 MPI\n'
                'tasks instead.')
            warnings.warn(message, RuntimeWarning)

    if mpi.rank == 0:
        if opt.directory is None:
            tmpdir = tempfile.mkdtemp(prefix='gpaw-test-')
        else:
            tmpdir = opt.directory
            if os.path.isdir(tmpdir):
                opt.keep_tmpdir = True
            else:
                os.mkdir(tmpdir)
    else:
        tmpdir = None
    tmpdir = mpi.broadcast_string(tmpdir)
    cwd = os.getcwd()
    os.chdir(tmpdir)
    operating_system = platform.system() + ' ' + platform.machine()
    operating_system += ' ' + ' '.join(platform.dist())
    python = platform.python_version() + ' ' + platform.python_compiler()
    python += ' ' + ' '.join(platform.architecture())
    if mpi.rank == 0:
        info()
        print('Running tests in', tmpdir)
        print('Jobs: {0}, Cores: {1}, debug-mode: {2}'
              .format(opt.jobs, mpi.size, debug))
    failed = TestRunner(tests, jobs=opt.jobs,
                        show_output=opt.show_output).run()
    os.chdir(cwd)
    if mpi.rank == 0:
        if len(failed) > 0:
            open('failed-tests.txt', 'w').write('\n'.join(failed) + '\n')
        elif not opt.keep_tmpdir:
            os.system('rm -rf ' + tmpdir)
    return len(failed)

# Backwards compatibility:
run = main

if __name__ == '__main__':
    main()



import os
import subprocess

from gpaw.test.big.agts import Cluster


class NiflheimCluster(Cluster):
    def __init__(self, asepath='', setuppath='$GPAW_SETUP_PATH'):
        self.asepath = asepath
        self.setuppath = setuppath
        
    def submit(self, job):
        dir = os.getcwd()
        os.chdir(job.dir)

        self.write_pylab_wrapper(job)

        gpaw_platform = os.environ['FYS_PLATFORM'].replace('-el6', '-2.6')

        if job.queueopts is None:
            if job.ncpus < 4:
                ppn = '%d:opteron4' % job.ncpus
                nodes = 1
                arch = 'linux-x86_64-x3455-2.6'
            elif job.ncpus % 16 == 0:
                ppn = '16:xeon16'
                nodes = job.ncpus // 16
                arch = 'linux-x86_64-sl230s-2.6'
            elif job.ncpus % 8 == 0:
                ppn = '8:xeon8'
                nodes = job.ncpus // 8
                arch = 'linux-x86_64-dl160g6-2.6'
            else:
                assert job.ncpus % 4 == 0
                ppn = '4:opteron4'
                nodes = job.ncpus // 4
                arch = 'linux-x86_64-x3455-2.6'
            queueopts = '-l nodes=%d:ppn=%s' % (nodes, ppn)
        else:
            queueopts = job.queueopts
            # the oldest, hopefully (?) common platform
            arch = 'linux-x86_64-x3455-2.6'

        gpaw_python = os.path.join(dir, 'gpaw', 'build',
                                   'bin.' + arch, 'gpaw-python')

        submit_pythonpath = ':'.join([
            self.asepath,
            dir + '/gpaw',
            '%s/gpaw/build/lib.%s' % (dir, arch),
            '$PYTHONPATH'])

        run_command = '. /home/opt/modulefiles/modulefiles_el6.sh&& '
        run_command += 'module load NUMPY/1.7.1-1&& '
        run_command += 'module load SCIPY/0.12.0-1&& '
        run_command += 'module load MATPLOTLIB/1.4.0-1&& '
        run_command += 'module load povray&& '
        run_command += 'module load bader&& '
        run_command += 'module load ABINIT&& '
        run_command += 'module load DACAPO&& '
        run_command += 'module load SCIENTIFICPYTHON&& '
        run_command += 'module load NWCHEM&& '
        run_command += 'module load AIMS&& '
        # force gpaw/gpaw/fftw.py to use the right libfftw3.so
        # see https://listserv.fysik.dtu.dk/pipermail/gpaw-developers/2012-July/003045.html
        if 0:  # libfftw3.so crashes
            run_command += 'module load intel-compilers&& '
            run_command += 'module load openmpi&& '  # intel version!
            fftw = 'fftw/3.3.3-' + gpaw_platform.replace('-2.6', '')
            fftw += '-tm-gfortran-openmpi-1.6.3-1'
            run_command += 'module load ' + fftw + '&& '
            run_command += 'export GPAW_FFTWSO="libfftw3.so"&& '
        if 0:  # libmkl_intel_lp64.so causes scipy crash?
            run_command += 'module unload fftw&& '  # all fftw must be unloaded!
            fftw = 'intel-mkl'
            run_command += 'module load ' + fftw + '&& '
            run_command += 'export GPAW_FFTWSO="libmkl_rt.so"&& '
        if 1: # numpy fftw has most chances to work
            run_command += 'module unload fftw&& '  # all fftw must be unloaded
            run_command += 'export GPAW_FFTWSO=""&& '  # and use numpy fftw
        # disable mpi_warn_on_fork - causes crashes on xeon16!
        run_command += 'export OMPI_MCA_mpi_warn_on_fork=0&& '

        if job.ncpus == 1:
            # don't use mpiexec here,
            # this allows one to start mpi inside the *.agts.py script
            run_command += 'module load intel-compilers&& '
            run_command += 'module load openmpi&& '
            run_command += ' export PYTHONPATH=' + submit_pythonpath + '&&'
            run_command += ' export GPAW_SETUP_PATH=' + self.setuppath + '&&'
            # we want to run GPAW/ASE scripts + gpaw-python with os.system!
            run_command += ' PATH=%s/ase/tools:$PATH' % dir + '&&'
            run_command += ' PATH=%s/gpaw/tools:$PATH' % dir + '&&'
            run_command += ' PATH=%s/gpaw/build/bin.%s:$PATH' % (dir, arch) + '&&'
            # we run other codes with asec
            run_command += ' export ASE_ABINIT_COMMAND="mpiexec abinit < PREFIX.files > PREFIX.log"&&'
            run_command += ' export ASE_AIMS_COMMAND="mpiexec ${AIMS_COMMAND} > aims.out"&&'
        else:
            run_command += 'module load intel-compilers&& '
            run_command += 'module load openmpi&& '
            run_command += 'mpiexec --mca mpi_paffinity_alone 1'
            run_command += ' -x PYTHONPATH=' + submit_pythonpath
            run_command += ' -x GPAW_SETUP_PATH=' + self.setuppath
            run_command += ' -x OMP_NUM_THREADS=1'

        p = subprocess.Popen(
            ['/usr/local/bin/qsub',
             '-V',
             queueopts,
             '-l',
             'walltime=%d:%02d:00' %
             (job.walltime // 3600, job.walltime % 3600 // 60),
             '-N',
             job.name],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE)

        out, err = p.communicate(
            'touch %s.start\n' % job.name +
            run_command +
            ' %s %s.py %s > %s.output\n' %
            (gpaw_python, job.script, job.args, job.name) +
            'echo $? > %s.done\n' % job.name)
        assert p.returncode == 0
        id = out.split('.')[0]
        job.pbsid = id
        os.chdir(dir)



from __future__ import print_function
from ase.lattice import bulk
from sys import argv
from ase.dft.kpoints import ibz_points, get_bandpath
from gpaw import *
from ase import *
from gpaw.test import gen
from gpaw import setup_paths

import os

"""
Calculate diamond with various parallelizations with GLLBSC
"""

xc = 'GLLBSC'
gen('C',xcname=xc)
setup_paths.insert(0, '.')

KSb = []
dxcb = []

eigensolver='rmm-diis'

for band in [1,2,4]:
    # Calculate ground state
    atoms = bulk('C', 'diamond', a=3.567)
    calc = GPAW(h=0.15, kpts=(4,4,4), xc=xc, nbands = 8,
                eigensolver=eigensolver,
                parallel={'band':band})
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    calc.write('Cgs.gpw')

    # Calculate accurate KS-band gap from band structure
    points = ibz_points['fcc']

    # CMB is in G-X
    G = points['Gamma']
    X = points['X']

    kpts, x, X = get_bandpath([G, X], atoms.cell, npoints=12)
    calc = GPAW('Cgs.gpw', kpts=kpts, fixdensity=True, symmetry='off',
                nbands=8, convergence=dict(bands=8), 
                eigensolver=eigensolver,
                parallel={'band':band})
    calc.get_atoms().get_potential_energy()
    # Get the accurate KS-band gap
    homolumo = calc.occupations.get_homo_lumo(calc.wfs)
    homo, lumo = homolumo
    print("band gap ",(lumo-homo)*27.2)
    
    # Redo the ground state calculation
    calc = GPAW(h=0.15, kpts=(4,4,4), xc=xc, nbands = 8, 
                eigensolver=eigensolver,
                parallel={'band':band})
    atoms.set_calculator(calc)
    atoms.get_potential_energy()
    # And calculate the discontinuity potential with accurate band gap
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    response.calculate_delta_xc(homolumo=homolumo)
    calc.write('CGLLBSC.gpw')

    # Redo the band structure calculation
    atoms, calc = restart('CGLLBSC.gpw', kpts=kpts, fixdensity=True,
                          symmetry='off', convergence=dict(bands=8))
    atoms.get_potential_energy()
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    KS, dxc = response.calculate_delta_xc_perturbation()
    KSb.append(KS)
    dxcb.append(dxc)    
    assert abs(KS+dxc-5.41)<0.10
    #M. Kuisma et. al, Phys. Rev. B 82, 115106, QP gap for C, 5.41eV, expt. 5.48eV

assert abs(KSb[0]-KSb[1])<1e-6
assert abs(KSb[0]-KSb[2])<1e-6
assert abs(dxcb[0]-dxcb[1])<1e-6
assert abs(dxcb[0]-dxcb[2])<1e-6



from __future__ import print_function
import numpy as np

from ase import Atoms, Atom
from ase.parallel import barrier, rank, size
from gpaw.cluster import Cluster
from gpaw.test import equal
from ase.structure import molecule
from math import pi, sqrt

R = 2.0
CO = Atoms([Atom('C', (1, 0, 0)), Atom('O', (1, 0, R))])

CO.rotate('y', pi/2)
equal(CO.positions[1, 0], R, 1e-10)

# translate
CO.translate(-CO.get_center_of_mass())
p = CO.positions.copy()
for i in range(2):
    equal(p[i, 1], 0, 1e-10)
    equal(p[i, 2], 0, 1e-10)

# rotate the nuclear axis to the direction (1,1,1)
CO.rotate(p[1] - p[0], (1, 1, 1))
q = CO.positions.copy()
for c in range(3):
    equal(q[0, c], p[0, 0] / sqrt(3), 1e-10)
    equal(q[1, c], p[1, 0] / sqrt(3), 1e-10)

# minimal box
b=4.0
CO = Cluster([Atom('C', (1, 0, 0)), Atom('O', (1, 0, R))])
CO.minimal_box(b)
cc = CO.get_cell() 
for c in range(3):
    width = 2*b
    if c==2:
        width += R
    equal(cc[c, c], width, 1e-10)

# minimal box, ensure multiple of 4
h = .13
b = [2, 3, 4]
CO.minimal_box(b, h=h)
cc = CO.get_cell() 
for c in range(3):
#    print "cc[c,c], cc[c,c] / h % 4 =", cc[c, c], cc[c, c] / h % 4
    for a in CO:
        print(a.symbol, b[c], a.position[c], cc[c, c] - a.position[c])
        assert(a.position[c] > b[c])
    equal(cc[c, c] / h % 4, 0.0, 1e-10)

# .............................................
# connected atoms
assert(len(CO.find_connected(0, 1.1 * R)) == 2)
assert(len(CO.find_connected(0, 0.9 * R)) == 1)

H2O = Cluster(molecule('H2O'))
assert (len(H2O.find_connected(0)) == 3)
assert (len(H2O.find_connected(0, scale=0.9)) == 1)

# .............................................
# I/O
fxyz='CO.xyz'
fpdb='CO.pdb'

cell = [2.,3.,R+2.]
CO.set_cell(cell, scale_atoms=True)
barrier()
CO.write(fxyz)
barrier()
CO_b = Cluster(filename=fxyz)
assert(len(CO) == len(CO_b))
#for a, b in zip(cell, CO_b.get_cell().diagonal()):
#    assert(a == b)
offdiagonal = CO_b.get_cell().sum() - CO_b.get_cell().diagonal().sum()
assert(offdiagonal == 0.0)
 
barrier()
CO.write(fpdb)

# read xyz files with additional info
read_with_additional = True
if read_with_additional:
    if rank == 0:
        f = open(fxyz, 'w')
        print("""2

C 0 0 0. 1 2 3
O 0 0 1. 6. 7. 8.""", file=f)
        f.close()

    barrier()

    CO = Cluster(filename=fxyz)



import os
import numpy as np
from ase import Atoms
from ase.lattice.hexagonal import Hexagonal
from gpaw import GPAW, FermiDirac
from gpaw.test import findpeak, equal
from gpaw.response.bse_new import BSE
from gpaw.mpi import rank

if 1:
    calc = GPAW(mode='pw',
                xc='GLLBSC',
                nbands=20,
                setups={'Mo': '6'},
                parallel={'band': 1, 'domain': 1},
                occupations=FermiDirac(0.001),
                convergence={'bands': -5},
                kpts=(9,9,1))
    
    a = 3.1604
    c = 10.0
    
    cell = Hexagonal(symbol='Mo', latticeconstant={'a':a, 'c':c}).get_cell()
    layer = Atoms(symbols='MoS2', cell=cell, pbc=(1,1,1),
                  scaled_positions=[(0, 0, 0),
                                    (2/3., 1/3., 0.3),
                                    (2/3., 1/3., -0.3)])
    
    pos = layer.get_positions()
    pos[1][2] = pos[0][2] + 3.172/2
    pos[2][2] = pos[0][2] - 3.172/2
    layer.set_positions(pos)
    layer.set_calculator(calc)
    layer.get_potential_energy()
    response = calc.hamiltonian.xc.xcs['RESPONSE']
    response.calculate_delta_xc()
    E_ks, dis = response.calculate_delta_xc_perturbation()
    calc.write('MoS2.gpw', mode='all')
else:
    dis = 0.5345

bse = BSE('MoS2.gpw',
          ecut=10,
          valence_bands=[8],
          conduction_bands=[9],
          eshift=dis,
          nbands=15,
          write_h=False,
          write_v=False,
          wfile=None,
          mode='BSE',
          truncation='2D',
          )
w_w, alpha_w = bse.get_polarizability(filename='bse.csv',
                                      pbc=[True, True, False],
                                      eta=0.02,
                                      w_w=np.linspace(0., 5., 5001),
                                      )
w_ = 0.833
I_ = 26.51
w, I = findpeak(w_w[:1000], alpha_w.imag[:1000])
equal(w, w_, 0.01) 
equal(I, I_, 0.1)

if rank == 0:
    os.remove('MoS2.gpw')
    os.remove('chi0.txt')
    os.remove('pair.txt')
